@@ -1,19 +1,19 @@
 '''
-Base classes for Covasim.
+Base classes for Covasim. These classes handle a lot of the boilerplate of the
+People and Sim classes (e.g. loading, saving, key lookups, etc.), so those classes
+can be focused on the disease-specific functionality.
 '''
 
-import datetime as dt
 import numpy as np
-import pylab as pl
 import pandas as pd
 import sciris as sc
+import datetime as dt
 from . import utils as cvu
 from . import misc as cvm
 from . import defaults as cvd
-from . import plotting as cvplt
 
 # Specify all externally visible classes this file defines
-__all__ = ['ParsObj', 'Result', 'BaseSim', 'BasePeople', 'Person', 'FlexDict', 'Contacts', 'Layer', 'TransTree']
+__all__ = ['ParsObj', 'Result', 'BaseSim', 'BasePeople', 'Person', 'FlexDict', 'Contacts', 'Layer']
 
 
 #%% Define simulation classes
@@ -155,6 +155,14 @@ class BaseSim(ParsObj):
         except: # If it's None or missing
             return 0
 
+    @property
+    def scaled_pop_size(self):
+        ''' Get the total population size, i.e. the number of agents times the scale factor -- if it fails, assume none '''
+        try:
+            return self['pop_size']*self['pop_scale']
+        except: # If it's None or missing
+            return 0
+
     @property
     def npts(self):
         ''' Count the number of time points '''
@@ -220,7 +228,7 @@ class BaseSim(ParsObj):
                         d = sc.readdate(d).date()
                     elif isinstance(d, dt.datetime):
                         d = d.date()
-                    d_day = (d - self['start_day']).days
+                    d_day = (d - cvm.date(self['start_day'])).days
                     days.append(d_day)
                 except Exception as E:
                     errormsg = f'Could not interpret "{d}" as a date: {str(E)}'
@@ -233,32 +241,40 @@ class BaseSim(ParsObj):
         return days
 
 
-    def date(self, ind, *args, dateformat=None):
+    def date(self, ind, *args, dateformat=None, as_date=False):
         '''
-        Convert an integer or list of integer simulation days to a date/list of dates.
+        Convert one or more integer days of simulation time to a date/list of dates --
+        by default returns a string, or returns a datetime Date object if as_date is True.
 
         Args:
             ind (int, list, or array): the day(s) in simulation time
+            as_date (bool): whether to return as a datetime date instead of a string
 
         Returns:
-            dates (str or list): the date relative to the simulation start day, as an integer
+            dates (str, Date, or list): the date(s) corresponding to the simulation day(s)
 
-        **Example**::
+        **Examples**::
 
-            sim.date(35) # Returns '2020-04-05'
+            sim.date(34) # Returns '2020-04-04'
+            sim.date([34, 54]) # Returns ['2020-04-04', '2020-04-24']
+            sim.date(34, 54, as_dt=True) # Returns [datetime.date(2020, 4, 4), datetime.date(2020, 4, 24)]
         '''
 
+        # Handle inputs
         if sc.isnumber(ind): # If it's a number, convert it to a list
             ind = sc.promotetolist(ind)
         ind.extend(args)
-
         if dateformat is None:
             dateformat = '%Y-%m-%d'
 
+        # Do the conversion
         dates = []
         for i in ind:
-            tmp = self['start_day'] + dt.timedelta(days=int(i))
-            dates.append(tmp.strftime(dateformat))
+            date_obj = cvm.date(self['start_day']) + dt.timedelta(days=int(i))
+            if as_date:
+                dates.append(date_obj)
+            else:
+                dates.append(date_obj.strftime(dateformat))
 
         # Return a string rather than a list if only one provided
         if len(ind)==1:
@@ -528,22 +544,19 @@ class BasePeople(sc.prettyobj):
     '''
     A class to handle all the boilerplate for people -- note that everything
     interesting happens in the People class.
+
+    Args:
+        pars (dict): a dictionary with, at minimum, keys 'pop_size' and 'n_days'
+
     '''
 
-    def __init__(self, pars=None, pop_size=None, *args, **kwargs):
+    def __init__(self, pars=None, **kwargs):
 
         # Handle pars and population size
+        pars = sc.mergedicts({'pop_size':0, 'n_days':0}, pars)
         self.pars = pars
-        if pop_size is None:
-            if pars is not None:
-                pop_size = pars['pop_size']
-            else:
-                pop_size = 0
-        pop_size = int(pop_size)
-        self.pop_size = pop_size
-        if pars is not None:
-            n_days = pars['n_days']
-        self.n_days = n_days
+        self.pop_size = pars['pop_size']
+        self.n_days = pars['n_days']
 
         # Other initialization
         self.t = 0 # Keep current simulation time
@@ -551,7 +564,7 @@ class BasePeople(sc.prettyobj):
         self.meta = cvd.PeopleMeta() # Store list of keys and dtypes
         self.contacts = None
         self.init_contacts() # Initialize the contacts
-        self.transtree = TransTree(pop_size=pop_size, n_days=n_days) # Initialize the transmission tree
+        self.infection_log = [] # Record of infections - keys for ['source','target','date','layer']
 
         return
 
@@ -575,7 +588,7 @@ class BasePeople(sc.prettyobj):
 
 
     def __len__(self):
-        ''' This is just a scalar, but validate() and resize() make sure it's right '''
+        ''' This is just a scalar, but validate() and _resize_arrays() make sure it's right '''
         return self.pop_size
 
 
@@ -678,9 +691,9 @@ class BasePeople(sc.prettyobj):
 
         # Check that the keys match
         contact_layer_keys = set(self.contacts.keys())
-        beta_layer_keys    = set(self.pars['beta_layer'].keys())
-        if contact_layer_keys != beta_layer_keys:
-            errormsg = f'Parameters layers {beta_layer_keys} are not consistent with contact layers {contact_layer_keys}'
+        layer_keys    = set(self.layer_keys())
+        if contact_layer_keys != layer_keys:
+            errormsg = f'Parameters layers {layer_keys} are not consistent with contact layers {contact_layer_keys}'
             raise ValueError(errormsg)
 
         # Check that the length of each array is consistent
@@ -694,11 +707,11 @@ class BasePeople(sc.prettyobj):
                 else:
                     if verbose:
                         print(f'Resizing "{key}" from {actual_len} to {expected_len}')
-                    self.resize(keys=key)
+                    self._resize_arrays(keys=key)
         return
 
 
-    def resize(self, pop_size=None, keys=None):
+    def _resize_arrays(self, pop_size=None, keys=None):
         ''' Resize arrays if any mismatches are found '''
         if pop_size is None:
             pop_size = len(self)
@@ -751,7 +764,7 @@ class BasePeople(sc.prettyobj):
         # Handle population size
         pop_size = len(people)
         if resize:
-            self.resize(pop_size=pop_size)
+            self._resize_arrays(pop_size=pop_size)
 
         # Iterate over people -- slow!
         for p,person in enumerate(people):
@@ -778,10 +791,9 @@ class BasePeople(sc.prettyobj):
     def add_contacts(self, contacts, lkey=None, beta=None):
         ''' Add new contacts to the array '''
 
+        # If no layer key is supplied and it can't be worked out from defaults, use the first layer
         if lkey is None:
             lkey = self.layer_keys()[0]
-        if lkey not in self.contacts:
-            self.contacts[lkey] = Layer()
 
         # Validate the supplied contacts
         if isinstance(contacts, Contacts):
@@ -810,8 +822,12 @@ class BasePeople(sc.prettyobj):
                 beta = cvd.default_float(beta)
                 new_layer['beta'] = np.ones(n, dtype=cvd.default_float)*beta
 
+            # Create the layer if it doesn't yet exist
+            if lkey not in self.contacts:
+                self.contacts[lkey] = Layer()
+
             # Actually include them, and update properties if supplied
-            for col in self.contacts[lkey].keys():
+            for col in self.contacts[lkey].keys(): # Loop over the supplied columns
                 self.contacts[lkey][col] = np.concatenate([self.contacts[lkey][col], new_layer[col]])
             self.contacts[lkey].validate()
 
@@ -824,23 +840,24 @@ class BasePeople(sc.prettyobj):
         into an edge list.
         '''
 
-        # Parse the list
+        # Handle layer keys
         lkeys = self.layer_keys()
+        if len(contacts):
+            contact_keys = contacts[0].keys() # Pull out the keys of this contact list
+            lkeys += [key for key in contact_keys if key not in lkeys] # Extend the layer keys
+
+        # Initialize the new contacts
         new_contacts = Contacts(layer_keys=lkeys)
         for lkey in lkeys:
             new_contacts[lkey]['p1']    = [] # Person 1 of the contact pair
             new_contacts[lkey]['p2']    = [] # Person 2 of the contact pair
 
-        try:
-            for p,cdict in enumerate(contacts):
-                for lkey,p_contacts in cdict.items():
-                    n = len(p_contacts) # Number of contacts
-                    new_contacts[lkey]['p1'].extend([p]*n) # e.g. [4, 4, 4, 4]
-                    new_contacts[lkey]['p2'].extend(p_contacts) # e.g. [243, 4538, 7,19]
-        except KeyError:
-            lkeystr = ', '.join(lkeys)
-            errormsg = f'Layer "{lkey}" could not be loaded since it was not among parameter keys "{lkeystr}". Please update manually or via sim.reset_layer_pars().'
-            raise sc.KeyNotFoundError(errormsg)
+        # Populate the new contacts
+        for p,cdict in enumerate(contacts):
+            for lkey,p_contacts in cdict.items():
+                n = len(p_contacts) # Number of contacts
+                new_contacts[lkey]['p1'].extend([p]*n) # e.g. [4, 4, 4, 4]
+                new_contacts[lkey]['p2'].extend(p_contacts) # e.g. [243, 4538, 7,19]
 
         # Turn into a dataframe
         for lkey in lkeys:
@@ -998,122 +1015,3 @@ class Layer(FlexDict):
             self[key] = df[key].to_numpy()
         return self
 
-
-
-class TransTree(sc.prettyobj):
-    '''
-    A class for holding a transmission tree. Sources and targets are both lists
-    of the same length as the population size. Sources has one entry for people who
-    have been infected, zero for those who haven't. Targets is a list of lists,
-    with the length of the list being the number of people that person infected.
-
-    Args:
-        pop_size (int): the number of people in the population
-    '''
-
-    def __init__(self, pop_size, n_days):
-        self.linelist = [None]*pop_size
-        self.targets  = [[] for p in range(len(self))] # Make a list of empty lists
-        self.detailed = None
-        self.pop_size = pop_size
-        self.n_days   = n_days
-        return
-
-
-    def __len__(self):
-        '''
-        The length of the transmission tree is the length of the line list,
-        which should equal the population size (non-infected people are None
-        in the line list).
-        '''
-        try:
-            return len(self.linelist)
-        except:
-            return 0
-
-
-    def make_targets(self, reset=False):
-        '''
-        Convert sources into targets -- same information, just grouped differently.
-        Usually done inside sim:step(), here just for completeness.
-        '''
-        if self.targets is None or reset:
-            self.targets = [[] for p in range(len(self))] # Make a list of empty lists
-            for transdict in self.linelist:
-                if transdict is not None:
-                    source = transdict['source']
-                    if source is not None: # e.g., from an importation
-                        self.targets[source].append(transdict)
-            return
-
-
-    def make_detailed(self, people, reset=False):
-        ''' Construct a detailed transmission tree, with additional information for each person '''
-        if self.detailed is None or reset:
-
-            # Reset to look like the line list, but with more detail
-            self.detailed = [None]*len(self)
-
-            for transdict in self.linelist:
-
-                if transdict is not None:
-
-                    # Pull out key quantities
-                    ddict  = sc.objdict(sc.dcp(transdict)) # For "detailed dictionary"
-                    source = ddict.source
-                    target = ddict.target
-                    ddict.s = sc.objdict() # Source
-                    ddict.t = sc.objdict() # Target
-
-                    # If the source is available (e.g. not a seed infection), loop over both it and the target
-                    if source is not None:
-                        stdict = {'s':source, 't':target}
-                    else:
-                        stdict = {'t':target}
-
-                    # Pull out each of the attributes relevant to transmission
-                    attrs = ['age', 'date_symptomatic', 'date_tested', 'date_diagnosed', 'date_quarantined', 'date_severe', 'date_critical', 'date_known_contact']
-                    for st,stind in stdict.items():
-                        for attr in attrs:
-                            ddict[st][attr] = people[attr][stind]
-                    if source is not None:
-                        for attr in attrs:
-                            if attr.startswith('date_'):
-                                is_attr = attr.replace('date_', 'is_') # Convert date to a boolean, e.g. date_diagnosed -> is_diagnosed
-                                ddict.s[is_attr] = ddict.s[attr] <= ddict['date'] # These don't make sense for people just infected (targets), only sources
-
-                        ddict.s.is_asymp   = np.isnan(people.date_symptomatic[source])
-                        ddict.s.is_presymp = ~ddict.s.is_asymp and ~ddict.s.is_symptomatic # Not asymptomatic and not currently symptomatic
-                    ddict.t['is_quarantined'] = ddict.t['date_quarantined'] <= ddict['date'] # This is the only target date that it makes sense to define since it can happen before infection
-
-                    self.detailed[target] = ddict
-
-        return
-
-
-    def plot(self, *args, **kwargs):
-        ''' Plot the transmission tree '''
-        return cvplt.plot_transtree(self, *args, **kwargs)
-
-
-    def animate(self, *args, **kwargs):
-        '''
-        Animate the transmission tree.
-
-        Args:
-            animate    (bool):  whether to animate the plot (otherwise, show when finished)
-            verbose    (bool):  print out progress of each frame
-            markersize (int):   size of the markers
-            sus_color  (list):  color for susceptibles
-            fig_args   (dict):  arguments passed to pl.figure()
-            axis_args  (dict):  arguments passed to pl.subplots_adjust()
-            plot_args  (dict):  arguments passed to pl.plot()
-            delay      (float): delay between frames in seconds
-            font_size  (int):   size of the font
-            colors     (list):  color of each person
-            cmap       (str):   colormap for each person (if colors is not supplied)
-
-        Returns:
-            fig: the figure object
-        '''
-        return cvplt.animate_transtree(self, *args, **kwargs)
\ No newline at end of file
