@@ -1,8 +1,13 @@
-import inspect
+'''
+Specify the core interventions available in Covasim. Other interventions can be
+defined by the user by inheriting from these classes.
+'''
+
 import numpy as np
 import pandas as pd
 import pylab as pl
 import sciris as sc
+import inspect
 import datetime as dt
 from . import utils as cvu
 from . import base as cvb
@@ -44,14 +49,23 @@ def InterventionDict(which, pars):
 
 class Intervention:
     '''
-    Abstract class for interventions
+    Base class for interventions. By default, interventions are printed using a
+    dict format, which they can be recreated from. To display all the attributes
+    of the intervention, use disp() instead.
+
+    Args:
+        label (str): a label for the intervention (used for plotting, and for ease of identification)
+        show_label (bool): whether or not to include the label, if provided, in the legend
+        do_plot (bool): whether or not to plot the intervention
+        line_args (dict): arguments passed to pl.axvline() whe plotting
     '''
-    def __init__(self, do_plot=None):
-        if do_plot is None:
-            do_plot = True
-        self.do_plot = do_plot
-        self.days = []
-        self.initialized = False
+    def __init__(self, label=None, show_label=True, do_plot=None, line_args=None):
+        self.label = label # e.g. "Close schools"
+        self.show_label = show_label # Show the label by default
+        self.do_plot = do_plot if do_plot is not None else True # Plot the intervention, including if None
+        self.line_args = sc.mergedicts(dict(linestyle='--', c=[0,0,0]), line_args) # Do not set alpha by default due to the issue of overlapping interventions
+        self.days = [] # The start and end days of the intervention
+        self.initialized = False # Whether or not it has been initialized
         return
 
 
@@ -67,6 +81,11 @@ class Intervention:
         return output
 
 
+    def disp(self):
+        ''' Print a detailed representation of the intervention '''
+        return print(sc.prepr(self))
+
+
     def _store_args(self):
         ''' Store the user-supplied arguments for later use in to_json '''
         f0 = inspect.currentframe() # This "frame", i.e. Intervention.__init__()
@@ -106,7 +125,7 @@ class Intervention:
         raise NotImplementedError
 
 
-    def plot(self, sim, ax=None):
+    def plot(self, sim, ax=None, **kwargs):
         '''
         Call function during plotting
 
@@ -116,16 +135,22 @@ class Intervention:
         Args:
             sim: the Sim instance
             ax: the axis instance
+            kwargs: passed to ax.axvline()
 
         Returns:
             None
         '''
+        line_args = sc.mergedicts(self.line_args, kwargs)
         if self.do_plot or self.do_plot is None:
             if ax is None:
                 ax = pl.gca()
             for day in self.days:
                 if day is not None:
-                    ax.axvline(day, linestyle='--', c=[0,0,0])
+                    if self.show_label: # Choose whether to include the label in the legend
+                        label = self.label
+                    else:
+                        label = None
+                    ax.axvline(day, label=label, **line_args)
         return
 
 
@@ -160,15 +185,16 @@ class dynamic_pars(Intervention):
 
     Args:
         pars (dict): described above
-        do_plot (bool): whether or not to plot the intervention
+        kwargs (dict): passed to Intervention()
 
     **Examples**::
 
-        interv = cv.dynamic_pars({'beta':{'days':[14, 28], 'vals':[0.005, 0.015]}, 'diag_factor':{'days':30, 'vals':0.0}}) # Change beta, and make diagnosed people stop transmitting
+        interv = cv.dynamic_pars({'beta':{'days':[14, 28], 'vals':[0.005, 0.015]}, 'rel_death_prob':{'days':30, 'vals':2.0}}) # Change beta, and make diagnosed people stop transmitting
     '''
 
-    def __init__(self, pars, do_plot=None):
-        super().__init__(do_plot=do_plot)
+    def __init__(self, pars, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
         subkeys = ['days', 'vals']
         for parkey in pars.keys():
             for subkey in subkeys:
@@ -182,7 +208,6 @@ class dynamic_pars(Intervention):
             if len_days != len_vals:
                 raise ValueError(f'Length of days ({len_days}) does not match length of values ({len_vals}) for parameter {parkey}')
         self.pars = pars
-        self._store_args()
         return
 
 
@@ -212,7 +237,7 @@ class sequence(Intervention):
     Args:
         days (list): the days on which to start applying each intervention
         interventions (list): the interventions to apply on those days
-        do_plot (bool): whether or not to plot the intervention
+        kwargs (dict): passed to Intervention()
 
     **Example**::
 
@@ -222,12 +247,12 @@ class sequence(Intervention):
                 ])
     '''
 
-    def __init__(self, days, interventions, do_plot=None):
-        super().__init__(do_plot=do_plot)
+    def __init__(self, days, interventions, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
         assert len(days) == len(interventions)
         self.days = days
         self.interventions = interventions
-        self._store_args()
         return
 
 
@@ -261,8 +286,7 @@ class change_beta(Intervention):
         days (int or array): the day or array of days to apply the interventions
         changes (float or array): the changes in beta (1 = no change, 0 = no transmission)
         layers (str or list): the layers in which to change beta
-        do_plot (bool): whether or not to plot the intervention
-
+        kwargs (dict): passed to Intervention()
 
     **Examples**::
 
@@ -270,13 +294,13 @@ class change_beta(Intervention):
         interv = cv.change_beta([14, 28], [0.7, 1], layers='s') # On day 14, reduce beta by 30%, and on day 28, return to 1 for schools
     '''
 
-    def __init__(self, days, changes, layers=None, do_plot=None):
-        super().__init__(do_plot=do_plot)
-        self.days = days
-        self.changes = sc.promotetoarray(changes)
-        self.layers = sc.promotetolist(layers, keepnone=True)
+    def __init__(self, days, changes, layers=None, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
+        self.days       = sc.dcp(days)
+        self.changes    = sc.dcp(changes)
+        self.layers     = sc.dcp(layers)
         self.orig_betas = None
-        self._store_args()
         return
 
 
@@ -288,15 +312,20 @@ class change_beta(Intervention):
             for d,day in enumerate(self.days):
                 self.days[d] = sim.day(day) # Ensure it's an integer and not a string or something
         self.days = sc.promotetoarray(self.days)
+
+        self.changes = sc.promotetoarray(self.changes)
         if len(self.days) != len(self.changes):
             errormsg = f'Number of days supplied ({len(self.days)}) does not match number of changes in beta ({len(self.changes)})'
             raise ValueError(errormsg)
+
         self.orig_betas = {}
+        self.layers = sc.promotetolist(self.layers, keepnone=True)
         for lkey in self.layers:
             if lkey is None:
                 self.orig_betas['overall'] = sim['beta']
             else:
                 self.orig_betas[lkey] = sim['beta_layer'][lkey]
+
         self.initialized = True
         return
 
@@ -325,7 +354,7 @@ class clip_edges(Intervention):
         start_day (int): the day to isolate contacts
         end_day (int): the day to end isolating contacts
         change (float or dict): the proportion of contacts to retain, a la change_beta (1 = no change, 0 = no transmission)
-        do_plot (bool): whether or not to plot the intervention
+        kwargs (dict): passed to Intervention()
 
     **Examples**::
 
@@ -333,15 +362,15 @@ class clip_edges(Intervention):
         interv = cv.clip_edges(start_day=25, end_day=35, change={'s':0.1}) # On day 25, remove 90% of school contacts, and on day 35, restore them
     '''
 
-    def __init__(self, start_day, change=None, end_day=None, verbose=False, do_plot=None):
-        super().__init__(do_plot=do_plot)
+    def __init__(self, start_day, change=None, end_day=None, verbose=False, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
         self.start_day  = start_day
         self.end_day    = end_day
         self.change     = change
         self.verbose    = verbose
         self.layer_keys = None
         self.contacts   = None
-        self._store_args()
         return
 
 
@@ -417,40 +446,106 @@ class clip_edges(Intervention):
 __all__+= ['test_num', 'test_prob', 'contact_tracing']
 
 
+# Process daily data
+def process_daily_data(daily_data, sim, start_day, as_int=False):
+    '''
+    This function performs one of two things: if the daily data are supplied as
+    a number, then it converts it to an array of the right length. If the daily
+    data are supplied as a Pandas series or dataframe with a date index, then it
+    reindexes it to match the start date of the simulation. Otherwise, it does
+    nothing.
+
+    Args:
+        daily_data (number, dataframe, or series): the data to convert to standardized format
+        sim (Sim): the simulation object
+        start_day (date): the start day of the simulation, in already-converted datetime.date format
+        as_int (bool): whether to convert to an integer
+    '''
+    if sc.isnumber(daily_data):  # If a number, convert to an array
+        if as_int: daily_data = int(daily_data) # Make it an integer
+        daily_data = np.array([daily_data] * sim.npts)
+    elif isinstance(daily_data, (pd.Series, pd.DataFrame)):
+        start_date = sim['start_day'] + dt.timedelta(days=start_day)
+        end_date = daily_data.index[-1]
+        dateindex = pd.date_range(start_date, end_date)
+        daily_data = daily_data.reindex(dateindex, fill_value=0).to_numpy()
+    return daily_data
+
+
+def get_subtargets(subtarget, sim):
+    '''
+    A small helper function to see if subtargeting is a list of indices to use,
+    or a function that needs to be called. If a function, it must take a single
+    argument, a sim object, and return a list of indices. Also validates the values.
+    Currently designed for use with testing interventions, but could be generalized
+    to other interventions.
+
+    Args:
+        subtarget (dict): dict with keys 'inds' and 'vals'; see test_num() for examples of a valid subtarget dictionary
+        sim (Sim): the simulation object
+    '''
+
+    # Validation
+    if 'inds' not in subtarget:
+        errormsg = f'The subtarget dict must have keys "inds" and "vals", but you supplied {subtarget}'
+        raise ValueError(errormsg)
+
+    # Handle the two options of type
+    if callable(subtarget['inds']): # A function has been provided
+        subtarget_inds = subtarget['inds'](sim) # Call the function to get the indices
+    else:
+        subtarget_inds = subtarget['inds'] # The indices are supplied directly
+
+    # Validate the values
+    subtarget_vals = subtarget['vals']
+    if sc.isiterable(subtarget_vals):
+        if len(subtarget_vals) != len(subtarget_inds):
+            errormsg = f'Length of subtargeting indices ({len(subtarget_inds)}) does not match length of values ({len(subtarget_vals)})'
+            raise ValueError(errormsg)
+
+
+    return subtarget_inds, subtarget_vals
+
+
 class test_num(Intervention):
     '''
     Test a fixed number of people per day.
 
     Args:
-        daily_tests (int or arr): number of tests per day; if integer, use that number every day
+        daily_tests (int or arr or dataframe/series): number of tests per day; if integer, use that number every day
         symp_test (float): odds ratio of a symptomatic person testing
         quar_test (float): probability of a person in quarantine testing
+        subtarget (dict or func): subtarget intervention to people with particular indices (format: {'ind': array of indices, or function to return indices from the sim, 'vals': value(s) to apply}
         sensitivity (float): test sensitivity
+        ili_prev (float or array or dataframe/series): Prevalence of influenza-like-illness symptoms in the population
         loss_prob (float): probability of the person being lost-to-follow-up
         test_delay (int): days for test result to be known
         start_day (int): day the intervention starts
         end_day (int): day the intervention ends
-        do_plot (bool): whether or not to plot the intervention
+        kwargs (dict): passed to Intervention()
 
-    **Example**::
+    **Examples**::
 
         interv = cv.test_num(daily_tests=[0.10*n_people]*npts)
-
-    Returns:
-        Intervention
+        interv = cv.test_num(daily_tests=[0.10*n_people]*npts, subtarget={'inds': sim.people.age>50, 'vals': 1.2}) # People over 50 are 20% more likely to test
+        interv = cv.test_num(daily_tests=[0.10*n_people]*npts, subtarget={'inds': lambda sim: sim.people.age>50, 'vals': 1.2}) # People over 50 are 20% more likely to test
     '''
 
-    def __init__(self, daily_tests, symp_test=100.0, quar_test=1.0, sensitivity=1.0, loss_prob=0, test_delay=0, start_day=0, end_day=None, do_plot=None):
-        super().__init__(do_plot=do_plot)
-        self.daily_tests = daily_tests #: Should be a list of length matching time
-        self.symp_test  = symp_test
+    def __init__(self, daily_tests, symp_test=100.0, quar_test=1.0, subtarget=None,
+                 ili_prev=None, sensitivity=1.0, loss_prob=0, test_delay=0,
+                 start_day=0, end_day=None, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
+        self.daily_tests = daily_tests # Should be a list of length matching time
+        self.symp_test   = symp_test   # Set probability of testing symptomatics
         self.quar_test   = quar_test
+        self.subtarget   = subtarget  # Set any other testing criteria
+        self.ili_prev    = ili_prev     # Should be a list of length matching time or a float or a dataframe
         self.sensitivity = sensitivity
         self.loss_prob   = loss_prob
         self.test_delay  = test_delay
         self.start_day   = start_day
         self.end_day     = end_day
-        self._store_args()
         return
 
 
@@ -462,14 +557,9 @@ class test_num(Intervention):
         self.end_day     = sim.day(self.end_day)
         self.days        = [self.start_day, self.end_day]
 
-        # Process daily tests -- has to be here rather than init so have access to the sim object
-        if sc.isnumber(self.daily_tests): # If a number, convert to an array
-            self.daily_tests = np.array([int(self.daily_tests)]*sim.npts)
-        elif isinstance(self.daily_tests, (pd.Series, pd.DataFrame)):
-            start_date = sim['start_day'] + dt.timedelta(days=self.start_day)
-            end_date = self.daily_tests.index[-1]
-            dateindex = pd.date_range(start_date, end_date)
-            self.daily_tests = self.daily_tests.reindex(dateindex, fill_value=0).to_numpy()
+        # Process daily data
+        self.daily_tests = process_daily_data(self.daily_tests, sim, self.start_day)
+        self.ili_prev    = process_daily_data(self.ili_prev,    sim, self.start_day)
 
         self.initialized = True
 
@@ -495,16 +585,32 @@ class test_num(Intervention):
         else:
             return
 
-        test_probs = np.ones(sim.n) # Begin by assigning equal tesitng probability to everyone
-        symp_inds  = cvu.true(sim.people.symptomatic)
-        quar_inds  = cvu.true(sim.people.quarantined)
-        diag_inds  = cvu.true(sim.people.diagnosed)
+        test_probs = np.ones(sim.n) # Begin by assigning equal testing probability to everyone
+
+        # Handle symptomatic testing, taking into account prevalence of ILI symptoms
+        symp_inds = cvu.true(sim.people.symptomatic)
+        if self.ili_prev is not None:
+            if rel_t < len(self.ili_prev):
+                n_ili = int(self.ili_prev[rel_t] * sim['pop_size'])  # Number with ILI symptoms on this day
+                ili_inds = cvu.choose(sim['pop_size'], n_ili) # Give some people some symptoms. Assuming that this is independent of COVID symptomaticity...
+                symp_inds = np.unique(np.concatenate((symp_inds, ili_inds)),0)
         test_probs[symp_inds] *= self.symp_test
+
+        # Handle quarantine testing
+        quar_inds  = cvu.true(sim.people.quarantined)
         test_probs[quar_inds] *= self.quar_test
+
+        # Handle any other user-specified testing criteria
+        if self.subtarget is not None:
+            subtarget_inds, subtarget_vals = get_subtargets(self.subtarget, sim)
+            test_probs[subtarget_inds] = test_probs[subtarget_inds]*subtarget_vals
+
+        # Don't re-diagnose people
+        diag_inds  = cvu.true(sim.people.diagnosed)
         test_probs[diag_inds] = 0.
 
+        # Now choose who gets tested and test them
         test_inds = cvu.choose_w(probs=test_probs, n=n_tests, unique=False)
-
         sim.people.test(test_inds, self.sensitivity, loss_prob=self.loss_prob, test_delay=self.test_delay)
 
         return
@@ -520,40 +626,46 @@ class test_prob(Intervention):
         asymp_prob (float): Probability of testing an asymptomatic (unquarantined) person
         symp_quar_prob (float): Probability of testing a symptomatic quarantined person
         asymp_quar_prob (float): Probability of testing an asymptomatic quarantined person
+        subtarget (dict): subtarget intervention to people with particular indices (see test_num() for details)
         test_sensitivity (float): Probability of a true positive
+        ili_prev (float or array): Prevalence of influenza-like-illness symptoms in the population
         loss_prob (float): Probability of loss to follow-up
         test_delay (int): How long testing takes
         start_day (int): When to start the intervention
+        kwargs (dict): passed to Intervention()
 
-    **Example**::
+    **Examples**::
 
         interv = cv.test_prob(symp_prob=0.1, asymp_prob=0.01) # Test 10% of symptomatics and 1% of asymptomatics
         interv = cv.test_prob(symp_quar_prob=0.4) # Test 40% of those in quarantine with symptoms
-
-    Returns:
-        Intervention
     '''
-    def __init__(self, symp_prob, asymp_prob=0.0, symp_quar_prob=None, asymp_quar_prob=None, test_sensitivity=1.0, loss_prob=0.0, test_delay=1, start_day=0, end_day=None, do_plot=None):
-        super().__init__(do_plot=do_plot)
+    def __init__(self, symp_prob, asymp_prob=0.0, symp_quar_prob=None, asymp_quar_prob=None, subtarget=None, ili_prev=None,
+                 test_sensitivity=1.0, loss_prob=0.0, test_delay=0, start_day=0, end_day=None, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
         self.symp_prob        = symp_prob
         self.asymp_prob       = asymp_prob
         self.symp_quar_prob   = symp_quar_prob  if  symp_quar_prob is not None else  symp_prob
         self.asymp_quar_prob  = asymp_quar_prob if asymp_quar_prob is not None else asymp_prob
+        self.subtarget        = subtarget
+        self.ili_prev         = ili_prev
         self.test_sensitivity = test_sensitivity
         self.loss_prob        = loss_prob
         self.test_delay       = test_delay
         self.start_day        = start_day
         self.end_day          = end_day
-        self._store_args()
         return
 
 
     def initialize(self, sim):
         ''' Fix the dates '''
-        self.start_day   = sim.day(self.start_day)
-        self.end_day     = sim.day(self.end_day)
-        self.days        = [self.start_day, self.end_day]
+        self.start_day = sim.day(self.start_day)
+        self.end_day   = sim.day(self.end_day)
+        self.days      = [self.start_day, self.end_day]
+        self.ili_prev  = process_daily_data(self.ili_prev, sim, self.start_day)
+
         self.initialized = True
+
         return
 
 
@@ -565,20 +677,37 @@ class test_prob(Intervention):
         elif self.end_day is not None and t > self.end_day:
             return
 
-        symp_inds       = cvu.true(sim.people.symptomatic)
-        asymp_inds      = cvu.false(sim.people.symptomatic)
+        # Define symptomatics, accounting for ILI prevalence
+        symp_inds  = cvu.true(sim.people.symptomatic)
+        if self.ili_prev is not None:
+            rel_t = t - self.start_day
+            if rel_t < len(self.ili_prev):
+                n_ili = int(self.ili_prev[rel_t] * sim['pop_size'])  # Number with ILI symptoms on this day
+                ili_inds = cvu.choose(sim['pop_size'], n_ili) # Give some people some symptoms, assuming that this is independent of COVID symptomaticity...
+                symp_inds = np.unique(np.concatenate((symp_inds, ili_inds)),0)
+
+        # Define asymptomatics: those who neither have COVID symptoms nor ILI symptoms
+        asymp_inds = np.setdiff1d(np.arange(sim['pop_size']), symp_inds)
+
+        # Handle quarantine and other testing criteria
         quar_inds       = cvu.true(sim.people.quarantined)
-        symp_quar_inds  = quar_inds[cvu.true(sim.people.symptomatic[quar_inds])]
-        asymp_quar_inds = quar_inds[cvu.false(sim.people.symptomatic[quar_inds])]
+        symp_quar_inds  = np.intersect1d(quar_inds, symp_inds)
+        asymp_quar_inds = np.intersect1d(quar_inds, asymp_inds)
+        if self.subtarget is not None:
+            subtarget_inds  = self.subtarget['inds']
         diag_inds       = cvu.true(sim.people.diagnosed)
 
-        test_probs = np.zeros(sim.n) # Begin by assigning equal tesitng probability to everyone
-        test_probs[symp_inds]       = self.symp_prob
-        test_probs[asymp_inds]      = self.asymp_prob
-        test_probs[symp_quar_inds]  = self.symp_quar_prob
-        test_probs[asymp_quar_inds] = self.asymp_quar_prob
-        test_probs[diag_inds]       = 0.
-        test_inds = cvu.binomial_arr(test_probs).nonzero()[0]
+        # Construct the testing probabilities piece by piece -- complicated, since need to do it in the right order
+        test_probs = np.zeros(sim.n) # Begin by assigning equal testing probability to everyone
+        test_probs[symp_inds]       = self.symp_prob       # People with symptoms
+        test_probs[asymp_inds]      = self.asymp_prob      # People without symptoms
+        test_probs[symp_quar_inds]  = self.symp_quar_prob  # People with symptoms in quarantine
+        test_probs[asymp_quar_inds] = self.asymp_quar_prob # People without symptoms in quarantine
+        if self.subtarget is not None:
+            subtarget_inds, subtarget_vals = get_subtargets(self.subtarget, sim)
+            test_probs[subtarget_inds] = subtarget_vals # People being explicitly subtargeted
+        test_probs[diag_inds] = 0.0 # People who are diagnosed don't test
+        test_inds = cvu.binomial_arr(test_probs).nonzero()[0] # Finally, calculate who actually tests
 
         sim.people.test(test_inds, test_sensitivity=self.test_sensitivity, loss_prob=self.loss_prob, test_delay=self.test_delay)
         sim.results['new_tests'][t] += int(len(test_inds)*sim['pop_scale']/sim.rescale_vec[t]) # If we're using dynamic scaling, we have to scale by pop_scale, not rescale_vec
@@ -591,25 +720,22 @@ class contact_tracing(Intervention):
     Contact tracing of positive people.
 
     Args:
-        trace_probs (float): probability of tracing, per layer
-        trace_time (int): days required to trace, per layer
-        start_day (int): intervention start day
-        end_day (int): intervention end day
-        contact_red (float): not implemented currently, but could potentially scale contact in this intervention
-        do_plot (bool): whether or not to plot
+        trace_probs (dict): probability of tracing, per layer
+        trace_time  (dict): days required to trace, per layer
+        start_day   (int):  intervention start day
+        end_day     (int):  intervention end day
+        test_delay  (int):  number of days a test result takes
+        presumptive (bool): whether or not to begin isolation and contact tracing on the presumption of a positive diagnosis
+        kwargs      (dict): passed to Intervention()
     '''
-    def __init__(self, trace_probs=None, trace_time=None, start_day=0, end_day=None, contact_red=None, do_plot=None):
-        super().__init__(do_plot=do_plot)
-        if trace_probs is None:
-            trace_probs = 1.0
-        if trace_time is None:
-            trace_time = 0.0
+    def __init__(self, trace_probs=None, trace_time=None, start_day=0, end_day=None, presumptive=False, **kwargs):
+        super().__init__(**kwargs) # Initialize the Intervention object
+        self._store_args() # Store the input arguments so the intervention can be recreated
         self.trace_probs = trace_probs
         self.trace_time  = trace_time
-        self.contact_red = contact_red
         self.start_day   = start_day
         self.end_day     = end_day
-        self._store_args()
+        self.presumptive = presumptive
         return
 
 
@@ -618,6 +744,10 @@ class contact_tracing(Intervention):
         self.start_day = sim.day(self.start_day)
         self.end_day   = sim.day(self.end_day)
         self.days      = [self.start_day, self.end_day]
+        if self.trace_probs is None:
+            self.trace_probs = 1.0
+        if self.trace_time is None:
+            self.trace_time = 0.0
         if sc.isnumber(self.trace_probs):
             val = self.trace_probs
             self.trace_probs = {k:val for k in sim.people.layer_keys()}
@@ -635,8 +765,14 @@ class contact_tracing(Intervention):
         elif self.end_day is not None and t > self.end_day:
             return
 
-        just_diagnosed_inds = cvu.true(sim.people.date_diagnosed == t) # Diagnosed this time step, time to trace
-        if len(just_diagnosed_inds): # If there are any just-diagnosed people, go trace their contacts
-            sim.people.trace(just_diagnosed_inds, self.trace_probs, self.trace_time)
+        # Figure out whom to test and trace
+        if not self.presumptive:
+            trace_from_inds = cvu.true(sim.people.date_diagnosed == t) # Diagnosed this time step, time to trace
+        else:
+            just_tested = cvu.true(sim.people.date_tested == t) # Tested this time step, time to trace
+            trace_from_inds = cvu.itruei(sim.people.exposed, just_tested) # This is necessary to avoid infinite chains of asymptomatic testing
+
+        if len(trace_from_inds): # If there are any just-diagnosed people, go trace their contacts
+            sim.people.trace(trace_from_inds, self.trace_probs, self.trace_time)
 
         return
