@@ -15,10 +15,16 @@ __all__ = ['People']
 class People(cvb.BasePeople):
     '''
     A class to perform all the operations on the people.
+
+    Args:
+        pars (dict): the sim parameters, e.g. sim.pars -- must have pop_size and n_days keys
+        strict (bool): whether or not to only create keys that are already in self.meta.person; otherwise, let any key be set
+        kwargs (dict): the actual data, e.g. from a popdict, being specified
+
     '''
 
-    def __init__(self, pars=None, pop_size=None, **kwargs):
-        super().__init__(pars, pop_size)
+    def __init__(self, pars, strict=True, **kwargs):
+        super().__init__(pars)
 
         # Set person properties -- mostly floats
         for key in self.meta.person:
@@ -42,11 +48,17 @@ class People(cvb.BasePeople):
         self._dtypes = {key:self[key].dtype for key in self.keys()} # Assign all to float by default
         self._lock = True # Stop further attributes from being set
 
+        # Store flows to be computed during simulation
+        self.flows = {key:0 for key in cvd.new_result_flows}
+
         # Set any values, if supplied
         if 'contacts' in kwargs:
             self.add_contacts(kwargs.pop('contacts'))
         for key,value in kwargs.items():
-            self.set(key, value)
+            if strict:
+                self.set(key, value)
+            else:
+                self[key] = value
 
         return
 
@@ -88,28 +100,26 @@ class People(cvb.BasePeople):
         self.is_exp = self.true('exposed') # For storing the interim values since used in every subsequent calculation
 
         # Perform updates
-        flows  = {key:0 for key in cvd.new_result_flows}
-        flows['new_infectious']  += self.check_infectious() # For people who are exposed and not infectious, check if they begin being infectious
-        flows['new_symptomatic'] += self.check_symptomatic()
-        flows['new_severe']      += self.check_severe()
-        flows['new_critical']    += self.check_critical()
-        flows['new_deaths']      += self.check_death()
-        flows['new_recoveries']  += self.check_recovery()
-        flows['new_diagnoses']   += self.check_diagnosed()
-        flows['new_quarantined'] += self.check_quar()
+        self.flows  = {key:0 for key in cvd.new_result_flows}
+        self.flows['new_infectious']  += self.check_infectious() # For people who are exposed and not infectious, check if they begin being infectious
+        self.flows['new_symptomatic'] += self.check_symptomatic()
+        self.flows['new_severe']      += self.check_severe()
+        self.flows['new_critical']    += self.check_critical()
+        self.flows['new_deaths']      += self.check_death()
+        self.flows['new_recoveries']  += self.check_recovery()
 
-        return flows
+        return
 
-
-    def update_states_post(self, flows):
+    def update_states_post(self):
         ''' Perform post-timestep updates '''
-        flows['new_diagnoses']   += self.check_diagnosed()
-        flows['new_quarantined'] += self.check_quar()
-        del self.is_exp # Tidy up
-        return flows
+        self.flows['new_diagnoses']   += self.check_diagnosed()
+        self.flows['new_quarantined'] += self.check_quar()
+        del self.is_exp  # Tidy up
+
+        return
 
 
-    def update_contacts(self, directed=False):
+    def update_contacts(self):
         ''' Refresh dynamic contacts, e.g. community '''
 
         # Figure out if anything needs to be done -- e.g. {'h':False, 'c':True}
@@ -123,9 +133,7 @@ class People(cvb.BasePeople):
             # Choose how many contacts to make
             pop_size   = len(self)
             n_contacts = self.pars['contacts'][lkey]
-            n_new = n_contacts*pop_size
-            if not directed:
-                n_new = int(n_new/2) # Since these get looped over in both directions later
+            n_new = int(n_contacts*pop_size/2) # Since these get looped over in both directions later
 
             # Create the contacts
             new_contacts = {} # Initialize
@@ -140,12 +148,6 @@ class People(cvb.BasePeople):
         return self.contacts
 
 
-    def make_detailed_transtree(self):
-        ''' Convenience function to avoid repeating the people object '''
-        self.transtree.make_detailed(self)
-        return
-
-
     #%% Methods for updating state
 
     def check_inds(self, current, date, filter_inds=None):
@@ -213,33 +215,46 @@ class People(cvb.BasePeople):
 
 
     def check_diagnosed(self):
-        ''' Check for new diagnoses '''
-        inds = self.check_inds(self.diagnosed, self.date_diagnosed, filter_inds=None)
-        self.diagnosed[inds] = True
-        return len(inds)
+        '''
+        Check for new diagnoses. Since most data are reported with diagnoses on
+        the date of the test, this function reports counts not for the number of
+        people who received a positive test result on a day, but rather, the number
+        of people who were tested on that day who are schedule to be diagnosed in
+        the future.
+        '''
+
+        # Handle people who tested today who will be diagnosed in future
+        test_pos_inds = self.check_inds(self.diagnosed, self.date_pos_test, filter_inds=None) # Find people who will be diagnosed in future
+        self.date_pos_test[test_pos_inds] = np.nan # Clear date of having will-be-positive test
+
+        # Handle people who were actually diagnosed today
+        diag_inds  = self.check_inds(self.diagnosed, self.date_diagnosed, filter_inds=None) # Find who was actually diagnosed on this timestep
+        self.diagnosed[diag_inds]   = True # Set these people to be diagnosed
+        self.quarantined[diag_inds] = False # If you are diagnosed, you are isolated, not in quarantine
+        self.date_end_quarantine[diag_inds] = np.nan # Clear end quarantine time
+
+        return len(test_pos_inds)
 
 
     def check_quar(self):
-        ''' Check for whether someone has been contacted by a positive'''
+        ''' Check for who gets put into quarantine'''
 
-        if self.pars['quar_period'] is not None:
-            not_diagnosed_inds = self.false('diagnosed')
-            all_inds = np.arange(len(self)) # Do dead people come out of quarantine?
+        not_diagnosed_inds = self.false('diagnosed')
+        all_inds = np.arange(len(self)) # Do dead people come out of quarantine?
 
-            # Perform quarantine - on all who have a date_known_contact (Filter to those not already diagnosed?)
-            inds = self.check_inds(self.quarantined, self.date_known_contact, filter_inds=not_diagnosed_inds) # Check who is quarantined, not_diagnosed_inds?
-            self.quarantine(inds) # Put people in quarantine
-            self.date_known_contact[inds] = np.nan # Clear date
+        # Perform quarantine - on all who have a date_known_contact (Filter to those not already diagnosed?)
+        not_quar_inds = self.check_inds(self.quarantined, self.date_known_contact, filter_inds=not_diagnosed_inds) # Check who is quarantined, not_diagnosed_inds?
+        not_recovered = cvu.ifalsei(self.recovered, not_quar_inds)  # Pull out people who are not recovered
+        quar_inds     = cvu.ifalsei(self.dead, not_recovered)       # ...or dead
+        self.quarantine(quar_inds) # Put people in quarantine
+        self.date_known_contact[quar_inds] = np.nan # Clear date
 
-            # Check for the end of quarantine - on all who are quarantined
-            end_inds = self.check_inds(~self.quarantined, self.date_end_quarantine, filter_inds=all_inds) # Note the double-negative here
-            self.quarantined[end_inds] = False # Release from quarantine
-            self.date_end_quarantine[end_inds] = np.nan # Clear end quarantine time
+        # Check for the end of quarantine - on all who are quarantined
+        end_inds = self.check_inds(~self.quarantined, self.date_end_quarantine, filter_inds=all_inds) # Note the double-negative here
+        self.quarantined[end_inds] = False # Release from quarantine
+        self.date_end_quarantine[end_inds] = np.nan # Clear end quarantine time
 
-            n_quarantined = len(inds)
-
-        else:
-            n_quarantined = 0
+        n_quarantined = len(quar_inds)
 
         return n_quarantined
 
@@ -262,7 +277,7 @@ class People(cvb.BasePeople):
         return
 
 
-    def infect(self, inds, bed_max=None, verbose=True):
+    def infect(self, inds, hosp_max=None, icu_max=None, source=None, layer=None):
         '''
         Infect people and determine their eventual outcomes.
             * Every infected person can infect other people, regardless of whether they develop symptoms
@@ -271,16 +286,28 @@ class People(cvb.BasePeople):
             * Critical cases either recover or die
 
         Args:
-            inds    (array):  array of people to infect
-            t       (int):    current timestep
-            bed_max (bool):   whether or not there is a bed available for this person
+            inds     (array): array of people to infect
+            hosp_max (bool):  whether or not there is an acute bed available for this person
+            icu_max  (bool):  whether or not there is an ICU bed available for this person
+            source   (array): source indices of the people who transmitted this infection (None if an importation or seed infection)
+            layer    (str):   contact layer this infection was transmitted on
 
         Returns:
             count (int): number of people infected
         '''
 
-        # Handle inputs
-        inds         = np.unique(inds[self.susceptible[inds]]) # Do not infect people who are not susceptible
+        # Remove duplicates
+        unique = np.unique(inds, return_index=True)[1]
+        inds = inds[unique]
+        if source is not None:
+            source = source[unique]
+
+        # Keep only susceptibles
+        keep = self.susceptible[inds] # Unique indices in inds and source that are also susceptible
+        inds = inds[keep]
+        if source is not None:
+            source = source[keep]
+
         n_infections = len(inds)
         durpars      = self.pars['dur']
 
@@ -288,6 +315,11 @@ class People(cvb.BasePeople):
         self.susceptible[inds]   = False
         self.exposed[inds]       = True
         self.date_exposed[inds]  = self.t
+        self.flows['new_infections'] += len(inds)
+
+        # Record transmissions
+        for i, target in enumerate(inds):
+            self.infection_log.append(dict(source=source[i] if source is not None else None, target=target, date=self.t, layer=layer))
 
         # Calculate how long before this person can infect other people
         self.dur_exp2inf[inds]     = cvu.sample(**durpars['exp2inf'], size=n_infections)
@@ -321,7 +353,7 @@ class People(cvb.BasePeople):
         # CASE 2.2: Severe cases: hospitalization required, may become critical
         self.dur_sym2sev[sev_inds] = cvu.sample(**durpars['sym2sev'], size=len(sev_inds)) # Store how long this person took to develop severe symptoms
         self.date_severe[sev_inds] = self.date_symptomatic[sev_inds] + self.dur_sym2sev[sev_inds]  # Date symptoms become severe
-        crit_probs = self.pars['rel_crit_prob'] * self.crit_prob[sev_inds] # Probability of these people being critical
+        crit_probs = self.pars['rel_crit_prob'] * self.crit_prob[sev_inds] * (self.pars['no_hosp_factor'] if hosp_max else 1.)# Probability of these people becoming critical - higher if no beds available
         is_crit = cvu.binomial_arr(crit_probs)  # See if they're a critical case
         crit_inds = sev_inds[is_crit]
         non_crit_inds = sev_inds[~is_crit]
@@ -334,7 +366,7 @@ class People(cvb.BasePeople):
         # CASE 2.2.2: Critical cases: ICU required, may die
         self.dur_sev2crit[crit_inds] = cvu.sample(**durpars['sev2crit'], size=len(crit_inds))
         self.date_critical[crit_inds] = self.date_severe[crit_inds] + self.dur_sev2crit[crit_inds]  # Date they become critical
-        death_probs = self.pars['rel_death_prob'] * self.death_prob[crit_inds] * (self.pars['OR_no_treat'] if bed_max else 1.) # Probability they'll die
+        death_probs = self.pars['rel_death_prob'] * self.death_prob[crit_inds] * (self.pars['no_icu_factor'] if icu_max else 1.) # Probability they'll die
         is_dead = cvu.binomial_arr(death_probs)  # Death outcome
         dead_inds = crit_inds[is_dead]
         alive_inds = crit_inds[~is_dead]
@@ -378,7 +410,9 @@ class People(cvb.BasePeople):
         not_lost      = cvu.n_binomial(1.0-loss_prob, len(not_diagnosed))
         final_inds    = not_diagnosed[not_lost]
 
+        # Store the date the person will be diagnosed, as well as the date they took the test which will come back positive
         self.date_diagnosed[final_inds] = self.t + test_delay
+        self.date_pos_test[final_inds] = self.t
 
         return
 
@@ -388,9 +422,10 @@ class People(cvb.BasePeople):
         Quarantine selected people starting on the current day. If a person is already
         quarantined, this will extend their quarantine.
         Args:
-            inds (array): indices of who to quarantine
+            inds (array): indices of who to quarantine, specified by check_quar()
         '''
         self.quarantined[inds] = True
+        self.date_quarantined[inds] = self.t
         self.date_end_quarantine[inds] = self.t + self.pars['quar_period']
         return
 
@@ -401,15 +436,14 @@ class People(cvb.BasePeople):
         Args:
             inds (array): indices of whose contacts to trace
             trace_probs (dict): probability of being able to trace people at each contact layer - should have the same keys as contacts
-            trace_time (dict): # days it'll take to trace people at each contact layer - should have the same keys as contacts
+            trace_time (dict): days it'll take to trace people at each contact layer - should have the same keys as contacts
         '''
-        # Figure out who has been contacted in the past
-        never_been_contacted = self.not_defined('date_known_contact')  # Indices of people who've never been contacted
 
         # Extract the indices of the people who'll be contacted
         traceable_layers = {k:v for k,v in trace_probs.items() if v != 0.} # Only trace if there's a non-zero tracing probability
         for lkey,this_trace_prob in traceable_layers.items():
             if self.pars['beta_layer'][lkey]: # Skip if beta is 0 for this layer
+                this_trace_time = trace_time[lkey]
 
                 # Find all the contacts of these people
                 nzinds = self.contacts[lkey]['beta'].nonzero()[0] # Find nonzero beta edges
@@ -420,20 +454,10 @@ class People(cvb.BasePeople):
                     inds_list.append(self.contacts[lkey][k2][nz_k1]) # Find their pairing partner
                 edge_inds = np.unique(np.concatenate(inds_list)) # Find all edges
 
-                # Check not diagnosed!
+                # Check contacts
                 contact_inds = cvu.binomial_filter(this_trace_prob, edge_inds) # Filter the indices according to the probability of being able to trace this layer
-                self.known_contact[contact_inds] = True
-
-                # Set the date of contact, careful not to override what might be an earlier date. TODO: this could surely be one operation?
-                first_time_contacted_inds   = np.intersect1d(never_been_contacted, contact_inds) # indices of people getting contacted for the first time
-                contacted_before_inds       = np.setdiff1d(contact_inds, first_time_contacted_inds) # indices of people who've been contacted before
-
-                if len(first_time_contacted_inds):
-                    self.date_known_contact[first_time_contacted_inds]  = self.t + trace_time[lkey] # Store when they were contacted
-                if len(contacted_before_inds):
-                    self.date_known_contact[contacted_before_inds]  = np.minimum(self.date_known_contact[contacted_before_inds], self.t + trace_time[lkey])
+                if len(contact_inds):
+                    self.known_contact[contact_inds] = True
+                    self.date_known_contact[contact_inds]  = np.fmin(self.date_known_contact[contact_inds], self.t+this_trace_time)
 
         return
-
-
-
