@@ -7,7 +7,6 @@ webapp.
 '''
 
 import numpy as np
-import pandas as pd
 import pylab as pl
 import sciris as sc
 import datetime as dt
@@ -16,35 +15,48 @@ import plotly.graph_objects as go
 from . import defaults as cvd
 
 
-__all__ = ['plot_sim', 'plot_scens', 'plot_result', 'plot_compare', 'plot_transtree', 'animate_transtree', 'plotly_sim', 'plotly_people', 'plotly_animate']
+__all__ = ['plot_sim', 'plot_scens', 'plot_result', 'plot_compare', 'plotly_sim', 'plotly_people', 'plotly_animate']
 
 
 #%% Plotting helper functions
 
-def handle_args(fig_args=None, plot_args=None, scatter_args=None, axis_args=None, fill_args=None, legend_args=None):
-    ''' Handle input arguments -- merge user input with defaults '''
+def handle_args(fig_args=None, plot_args=None, scatter_args=None, axis_args=None, fill_args=None, legend_args=None, show_args=None):
+    ''' Handle input arguments -- merge user input with defaults; see sim.plot for documentation '''
     args = sc.objdict()
     args.fig     = sc.mergedicts({'figsize': (16, 14)}, fig_args)
     args.plot    = sc.mergedicts({'lw': 3, 'alpha': 0.7}, plot_args)
-    args.scatter = sc.mergedicts({'s':70, 'marker':'s'}, scatter_args)
+    args.scatter = sc.mergedicts({'s':70, 'marker':'s', 'alpha':0.7, 'zorder':0}, scatter_args)
     args.axis    = sc.mergedicts({'left': 0.10, 'bottom': 0.05, 'right': 0.95, 'top': 0.97, 'wspace': 0.25, 'hspace': 0.25}, axis_args)
     args.fill    = sc.mergedicts({'alpha': 0.2}, fill_args)
-    args.legend  = sc.mergedicts({'loc': 'best'}, legend_args)
+    args.legend  = sc.mergedicts({'loc': 'best', 'frameon':False}, legend_args)
+    args.show    = sc.mergedicts({'data':True, 'interventions':True, 'legend':True, }, show_args)
+
+    # Handle what to show
+    show_keys = ['uncertainty', 'data', 'ticks', 'interventions', 'legend']
+    args.show = {k:True for k in show_keys}
+    if show_args in [True, False]: # Handle all on or all off
+        args.show = {k:show_args for k in show_keys}
+    else:
+        args.show = sc.mergedicts(args.show, show_args)
+
     return args
 
 
 def handle_to_plot(which, to_plot, n_cols, sim):
     ''' Handle which quantities to plot '''
 
-    if to_plot is None:
+    # If not specified or specified as a string, load defaults
+    if to_plot is None or isinstance(to_plot, str):
         if which == 'sim':
-            to_plot = cvd.get_sim_plots()
+            to_plot = cvd.get_sim_plots(to_plot)
         elif which =='scens':
-            to_plot = cvd.get_scen_plots()
+            to_plot = cvd.get_scen_plots(to_plot)
         else:
             errormsg = f'"which" must be "sim" or "scens", not "{which}"'
             raise NotImplementedError(errormsg)
-    elif isinstance(to_plot, list): # If a list of keys has been supplied
+
+    # If a list of keys has been supplied
+    if isinstance(to_plot, list):
         to_plot_list = to_plot # Store separately
         to_plot = sc.odict() # Create the dict
         for reskey in to_plot_list:
@@ -107,13 +119,13 @@ def create_subplots(figs, fig, shareax, n_rows, n_cols, pnum, fig_args, sep_figs
     return ax
 
 
-def plot_data(sim, key, scatter_args):
+def plot_data(sim, ax, key, scatter_args):
     ''' Add data to the plot '''
     if sim.data is not None and key in sim.data and len(sim.data[key]):
         this_color = sim.results[key].color
         data_t = (sim.data.index-sim['start_day'])/np.timedelta64(1,'D') # Convert from data date to model output index based on model start date
-        pl.scatter(data_t, sim.data[key], c=[this_color], **scatter_args)
-        pl.scatter(pl.nan, pl.nan, c=[(0,0,0)], label='Data', **scatter_args)
+        ax.scatter(data_t, sim.data[key], c=[this_color], **scatter_args)
+        ax.scatter(pl.nan, pl.nan, c=[(0,0,0)], label='Data', **scatter_args)
     return
 
 
@@ -205,13 +217,14 @@ def tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, default_name='covas
 #%% Core plotting functions
 
 def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
-         scatter_args=None, axis_args=None, fill_args=None, legend_args=None, as_dates=True, dateformat=None,
-         interval=None, n_cols=1, font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
-         log_scale=False, colors=None, labels=None, do_show=True, sep_figs=False, fig=None):
+         scatter_args=None, axis_args=None, fill_args=None, legend_args=None, show_args=None,
+         as_dates=True, dateformat=None, interval=None, n_cols=1, font_size=18, font_family=None,
+         grid=False, commaticks=True, setylim=True, log_scale=False, colors=None, labels=None,
+         do_show=True, sep_figs=False, fig=None):
     ''' Plot the results of a single simulation -- see Sim.plot() for documentation '''
 
     # Handle inputs
-    args = handle_args(fig_args, plot_args, scatter_args, axis_args, fill_args, legend_args)
+    args = handle_args(fig_args, plot_args, scatter_args, axis_args, fill_args, legend_args, show_args)
     to_plot, n_rows = handle_to_plot('sim', to_plot, n_cols, sim=sim)
     fig, figs, ax = create_figs(args, font_size, font_family, sep_figs, fig)
 
@@ -229,21 +242,26 @@ def plot_sim(sim, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot
                 label = labels[reskey]
             else:
                 label = res.name
-            if res.low is not None and res.high is not None:
+            if res.low is not None and res.high is not None and args.show['uncertainty']:
                 ax.fill_between(res_t, res.low, res.high, color=color, **args.fill) # Create the uncertainty bound
-            ax.plot(res_t, res.values, label=label, **args.plot, c=color)
-            plot_data(sim, reskey, args.scatter) # Plot the data
-            reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
-        plot_interventions(sim, ax) # Plot the interventions
-        title_grid_legend(ax, title, grid, commaticks, setylim, args.legend) # Configure the title, grid, and legend
+            ax.plot(res_t, res.values, label=label, **args.plot, c=color) # Actually plot the sim!
+            if args.show['data']:
+                plot_data(sim, ax, reskey, args.scatter) # Plot the data
+            if args.show['ticks']:
+                reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
+        if args.show['interventions']:
+            plot_interventions(sim, ax) # Plot the interventions
+        if args.show['legend']:
+            title_grid_legend(ax, title, grid, commaticks, setylim, args.legend) # Configure the title, grid, and legend
 
     return tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, default_name='covasim.png')
 
 
 def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None, plot_args=None,
-         scatter_args=None, axis_args=None, fill_args=None, legend_args=None, as_dates=True, dateformat=None,
-         interval=None, n_cols=1, font_size=18, font_family=None, grid=False, commaticks=True, setylim=True,
-         log_scale=False, colors=None, labels=None, do_show=True, sep_figs=False, fig=None):
+         scatter_args=None, axis_args=None, fill_args=None, legend_args=None, show_args=None,
+         as_dates=True, dateformat=None, interval=None, n_cols=1, font_size=18, font_family=None,
+         grid=False, commaticks=True, setylim=True, log_scale=False, colors=None, labels=None,
+         do_show=True, sep_figs=False, fig=None):
     ''' Plot the results of a scenario -- see Scenarios.plot() for documentation '''
 
     # Handle inputs
@@ -269,12 +287,17 @@ def plot_scens(scens, to_plot=None, do_save=None, fig_path=None, fig_args=None,
                     label = labels[scenkey]
                 else:
                     label = scendata.name
-                ax.fill_between(scens.tvec, scendata.low, scendata.high, color=color, **args.fill) # Create the uncertainty bound
+                if args.show['uncertainty']:
+                    ax.fill_between(scens.tvec, scendata.low, scendata.high, color=color, **args.fill) # Create the uncertainty bound
                 ax.plot(scens.tvec, res_y, label=label, c=color, **args.plot) # Plot the actual line
-                plot_data(sim, reskey, args.scatter) # Plot the data
-                plot_interventions(sim, ax) # Plot the interventions
-                reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
-        title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, pnum==0) # Configure the title, grid, and legend -- only show legend for first
+                if args.show['data']:
+                    plot_data(sim, ax, reskey, args.scatter) # Plot the data
+                if args.show['interventions']:
+                    plot_interventions(sim, ax) # Plot the interventions
+                if args.show['ticks']:
+                    reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
+        if args.show['legend']:
+            title_grid_legend(ax, title, grid, commaticks, setylim, args.legend, pnum==0) # Configure the title, grid, and legend -- only show legend for first
 
     return tidy_up(fig, figs, sep_figs, do_save, fig_path, do_show, default_name='covasim_scenarios.png')
 
@@ -293,7 +316,6 @@ def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter
     # Gather results
     res = sim.results[key]
     res_t = sim.results['t']
-    res_y = res.values
     if color is None:
         color = res.color
 
@@ -309,8 +331,10 @@ def plot_result(sim, key, fig_args=None, plot_args=None, axis_args=None, scatter
     # Do the plotting
     if label is None:
         label = res.name
-    ax.plot(res_t, res_y, c=color, label=label, **args.plot)
-    plot_data(sim, key, args.scatter) # Plot the data
+    if res.low is not None and res.high is not None:
+        ax.fill_between(res_t, res.low, res.high, color=color, **args.fill) # Create the uncertainty bound
+    ax.plot(res_t, res.values, c=color, label=label, **args.plot)
+    plot_data(sim, ax, key, args.scatter) # Plot the data
     plot_interventions(sim, ax) # Plot the interventions
     title_grid_legend(ax, res.name, grid, commaticks, setylim, args.legend) # Configure the title, grid, and legend
     reset_ticks(ax, sim, interval, as_dates) # Optionally reset tick marks (useful for e.g. plotting weeks/months)
@@ -362,183 +386,6 @@ def plot_compare(df, log_scale=True, fig_args=None, plot_args=None, axis_args=No
     return fig
 
 
-#%% Transtree functions
-def plot_transtree(tt, *args, **kwargs):
-    ''' Plot the transmission tree; see TransTree.plot() for documentation '''
-
-    fig_args = kwargs.get('fig_args', dict(figsize=(16,10)))
-
-    if tt.detailed is None:
-        errormsg = 'Please run sim.people.make_detailed_transtree() before calling plotting'
-        raise ValueError(errormsg)
-
-    ttlist = []
-    for entry in tt.detailed:
-        if entry and entry.source:
-            tdict = {
-                'date': entry.date,
-                'layer': entry.layer,
-                's_asymp': entry.s.is_asymp,
-                's_presymp': entry.s.is_presymp,
-                's_sev': entry.s.is_severe,
-                's_crit': entry.s.is_critical,
-                's_diag': entry.s.is_diagnosed,
-                's_quar': entry.s.is_quarantined,
-                't_quar': entry.t.is_quarantined,
-                     }
-            ttlist.append(tdict)
-
-    df = pd.DataFrame(ttlist).rename(columns={'date': 'Day'})
-    df = df.loc[df['layer'] != 'seed_infection']
-
-    df['Stage'] = 'Symptomatic'
-    df.loc[df['s_asymp'], 'Stage'] = 'Asymptomatic'
-    df.loc[df['s_presymp'], 'Stage'] = 'Presymptomatic'
-
-    df['Severity'] = 'Mild'
-    df.loc[df['s_sev'], 'Severity'] = 'Severe'
-    df.loc[df['s_crit'], 'Severity'] = 'Critical'
-
-    fig = pl.figure(**fig_args)
-    i=1; r=2; c=3
-
-    def plot(key, title, i):
-        dat = df.groupby(['Day', key]).size().unstack(key)
-        ax = pl.subplot(r,c,i);
-        dat.plot(ax=ax, legend=None)
-        pl.legend(title=None)
-        ax.set_title(title)
-
-    to_plot = {
-        'layer':'Layer',
-        'Stage':'Source stage',
-        's_diag':'Source diagnosed',
-        's_quar':'Source quarantined',
-        't_quar':'Target quarantined',
-        'Severity':'Symptomatic source severity'
-    }
-    for i, (key, title) in enumerate(to_plot.items()):
-        plot(key, title, i+1)
-
-    return fig
-
-
-def animate_transtree(tt, **kwargs):
-    ''' Plot an animation of the transmission tree; see TransTree.animate() for documentation '''
-
-    # Settings
-    animate    = kwargs.get('animate', True)
-    verbose    = kwargs.get('verbose', False)
-    msize      = kwargs.get('markersize', 10)
-    sus_color  = kwargs.get('sus_color', [0.5, 0.5, 0.5])
-    fig_args   = kwargs.get('fig_args', dict(figsize=(24,16)))
-    axis_args  = kwargs.get('axis_args', dict(left=0.10, bottom=0.05, right=0.85, top=0.97, wspace=0.25, hspace=0.25))
-    plot_args  = kwargs.get('plot_args', dict(lw=2, alpha=0.5))
-    delay      = kwargs.get('delay', 0.2)
-    font_size  = kwargs.get('font_size', 18)
-    colors     = kwargs.get('colors', None)
-    cmap       = kwargs.get('cmap', 'parula')
-    pl.rcParams['font.size'] = font_size
-    if colors is None:
-        colors = sc.vectocolor(tt.pop_size, cmap=cmap)
-
-    # Initialization
-    n = tt.n_days + 1
-    frames = [list() for i in range(n)]
-    tests  = [list() for i in range(n)]
-    diags  = [list() for i in range(n)]
-    quars  = [list() for i in range(n)]
-
-    # Construct each frame of the animation
-    for i,entry in enumerate(tt.detailed): # Loop over every person
-        frame = sc.objdict()
-        tdq = sc.objdict() # Short for "tested, diagnosed, or quarantined"
-
-        # This person became infected
-        if entry:
-            source = entry['source']
-            target = entry['target']
-            target_date = entry['date']
-            if source: # Seed infections and importations won't have a source
-                source_date = tt.detailed[source]['date']
-            else:
-                source = 0
-                source_date = 0
-
-            # Construct this frame
-            frame.x = [source_date, target_date]
-            frame.y = [source, target]
-            frame.c = colors[source]
-            frame.i = True # If this person is infected
-            frames[target_date].append(frame)
-
-            # Handle testing, diagnosis, and quarantine
-            tdq.t = target
-            tdq.d = target_date
-            tdq.c = colors[target]
-            date_t = entry.t.date_tested
-            date_d = entry.t.date_diagnosed
-            date_q = entry.t.date_known_contact
-            if ~np.isnan(date_t) and date_t<n: tests[int(date_t)].append(tdq)
-            if ~np.isnan(date_d) and date_d<n: diags[int(date_d)].append(tdq)
-            if ~np.isnan(date_q) and date_q<n: quars[int(date_q)].append(tdq)
-
-        # This person did not become infected
-        else:
-            frame.x = [0]
-            frame.y = [i]
-            frame.c = sus_color
-            frame.i = False
-            frames[0].append(frame)
-
-    # Configure plotting
-    fig = pl.figure(**fig_args)
-    pl.subplots_adjust(**axis_args)
-    ax = fig.add_subplot(1,1,1)
-
-    # Create the legend
-    ax2 = pl.axes([0.85, 0.05, 0.14, 0.9])
-    ax2.axis('off')
-    lcol = colors[0]
-    na = np.nan # Shorten
-    pl.plot(na, na, '-', c=lcol, **plot_args, label='Transmission')
-    pl.plot(na, na, 'o', c=lcol, markersize=msize, **plot_args, label='Source')
-    pl.plot(na, na, '*', c=lcol, markersize=msize, **plot_args, label='Target')
-    pl.plot(na, na, 'o', c=lcol, markersize=msize*2, fillstyle='none', **plot_args, label='Tested')
-    pl.plot(na, na, 's', c=lcol, markersize=msize*1.2, **plot_args, label='Diagnosed')
-    pl.plot(na, na, 'x', c=lcol, markersize=msize*2.0, label='Known contact')
-    pl.legend()
-
-    # Plot the animation
-    pl.sca(ax)
-    for day in range(n):
-        pl.title(f'Day: {day}')
-        pl.xlim([0, n])
-        pl.ylim([0, tt.pop_size])
-        pl.xlabel('Day')
-        pl.ylabel('Person')
-        flist = frames[day]
-        tlist = tests[day]
-        dlist = diags[day]
-        qlist = quars[day]
-        if verbose: print(i, flist)
-        for f in flist:
-            if verbose: print(f)
-            pl.plot(f.x[0], f.y[0], 'o', c=f.c, markersize=msize, **plot_args) # Plot sources
-            pl.plot(f.x, f.y, '-', c=f.c, **plot_args) # Plot transmission lines
-            if f.i: # If this person is infected
-                pl.plot(f.x[1], f.y[1], '*', c=f.c, markersize=msize, **plot_args) # Plot targets
-        for tdq in tlist: pl.plot(tdq.d, tdq.t, 'o', c=tdq.c, markersize=msize*2, fillstyle='none') # Tested; No alpha for this
-        for tdq in dlist: pl.plot(tdq.d, tdq.t, 's', c=tdq.c, markersize=msize*1.2, **plot_args) # Diagnosed
-        for tdq in qlist: pl.plot(tdq.d, tdq.t, 'x', c=tdq.c, markersize=msize*2.0) # Quarantine; no alpha for this
-        pl.plot([0, day], [0.5, 0.5], c='k', lw=5) # Plot the endless march of time
-        if animate: # Whether to animate
-            pl.pause(delay)
-
-    return fig
-
-
-
 #%% Plotly functions
 
 def get_individual_states(sim):
@@ -589,18 +436,20 @@ def get_individual_states(sim):
 plotly_legend = dict(legend_orientation="h", legend=dict(x=0.0, y=1.18))
 
 
-def plotly_interventions(sim, fig):
+def plotly_interventions(sim, fig, add_to_legend=False):
     ''' Add vertical lines for interventions to the plot '''
     if sim['interventions']:
         for interv in sim['interventions']:
             if hasattr(interv, 'days'):
                 for interv_day in interv.days:
                     if interv_day and interv_day < sim['n_days']:
-                        fig.add_shape(dict(type="line", xref="x", yref="paper", x0=interv_day, x1=interv_day, y0=0, y1=1, name='Intervention', line=dict(width=0.5, dash='dash')))
-                        fig.update_layout(annotations=[dict(x=interv_day, y=1.07, xref="x", yref="paper", text="Intervention change", showarrow=False)])
+                        interv_date = sim.date(interv_day, as_date=True)
+                        fig.add_shape(dict(type="line", xref="x", yref="paper", x0=interv_date, x1=interv_date, y0=0, y1=1, line=dict(width=0.5, dash='dash')))
+                        if add_to_legend:
+                            fig.add_trace(go.Scatter(x=[interv_date], y=[0], mode='lines', name='Intervention change', line=dict(width=0.5, dash='dash')))
     return
 
-def plotly_sim(sim):
+def plotly_sim(sim, do_show=False):
     ''' Main simulation results -- parallel of sim.plot() '''
 
     plots = []
@@ -618,10 +467,15 @@ def plotly_sim(sim):
                 ydata = sim.data[key]
                 fig.add_trace(go.Scatter(x=xdata, y=ydata, mode='markers', name=label + ' (data)', line_color=this_color))
 
-        plotly_interventions(sim, fig)
+        plotly_interventions(sim, fig, add_to_legend=(p==0)) # Only add the intervention label to the legend for the first plot
         fig.update_layout(title={'text':title}, yaxis_title='Count', autosize=True, **plotly_legend)
 
         plots.append(fig)
+
+    if do_show:
+        for fig in plots:
+            fig.show()
+
     return plots
 
 
@@ -774,4 +628,4 @@ def plotly_animate(sim, do_show=False):
     if do_show:
         fig.show()
 
-    return fig
\ No newline at end of file
+    return fig
