@@ -5,13 +5,13 @@ Defines functions for making the population.
 #%% Imports
 import numpy as np # Needed for a few things not provided by pl
 import sciris as sc
+from collections import defaultdict
 from . import requirements as cvreq
 from . import utils as cvu
 from . import data as cvdata
 from . import defaults as cvd
 from . import parameters as cvpars
 from . import people as cvppl
-from collections import defaultdict
 
 
 # Specify all externally visible functions this file defines
@@ -60,14 +60,16 @@ def make_people(sim, save_pop=False, popfile=None, verbose=None, die=True, reset
     # Actually create the population
     if sim.popdict and not reset:
         popdict = sim.popdict # Use stored one
-        layer_keys = list(popdict['contacts'][0].keys()) # Assume there's at least one contact!
         sim.popdict = None # Once loaded, remove
     else:
         # Create the population
         if pop_type in ['random', 'clustered', 'hybrid']:
-            popdict, layer_keys = make_randpop(sim, microstructure=pop_type)
+            popdict = make_randpop(sim, microstructure=pop_type)
         elif pop_type == 'synthpops':
-            popdict, layer_keys = make_synthpop(sim)
+            popdict = make_synthpop(sim)
+        elif pop_type is None:
+            errormsg = f'You have set pop_type=None. This is fine, but you must ensure sim.popdict exists before calling make_people().'
+            raise ValueError(errormsg)
         else:
             errormsg = f'Population type "{pop_type}" not found; choices are random, clustered, hybrid, or synthpops'
             raise ValueError(errormsg)
@@ -77,9 +79,7 @@ def make_people(sim, save_pop=False, popfile=None, verbose=None, die=True, reset
         sim['prognoses'] = cvpars.get_prognoses(sim['prog_by_age'])
 
     # Actually create the people
-    sim.layer_keys = layer_keys
-    people = cvppl.People(sim.pars, **popdict) # List for storing the people
-    sim.people = people
+    people = cvppl.People(sim.pars, uid=popdict['uid'], age=popdict['age'], sex=popdict['sex'], contacts=popdict['contacts']) # List for storing the people
 
     average_age = sum(popdict['age']/pop_size)
     sc.printv(f'Created {pop_size} people, average age {average_age:0.2f} years', 2, verbose)
@@ -94,11 +94,31 @@ def make_people(sim, save_pop=False, popfile=None, verbose=None, die=True, reset
             if verbose:
                 print(f'Saved population of type "{pop_type}" with {pop_size:n} people to {filepath}')
 
-    return
+    return people
 
 
 def make_randpop(sim, use_age_data=True, use_household_data=True, sex_ratio=0.5, microstructure=False):
-    ''' Make a random population, without contacts '''
+    '''
+    Make a random population, with contacts.
+
+    This function returns a "popdict" dictionary, which has the following (required) keys:
+
+        - uid: an array of (usually consecutive) integers of length N, uniquely identifying each agent
+        - age: an array of floats of length N, the age in years of each agent
+        - sex: an array of integers of length N (not currently used, so does not have to be binary)
+        - contacts: list of length N listing the contacts; see make_random_contacts() for details
+        - layer_keys: a list of strings representing the different contact layers in the population; see make_random_contacts() for details
+
+    Args:
+        sim (Sim): the simulation object
+        use_age_data (bool): whether to use location-specific age data
+        use_household_data (bool): whether to use location-specific household size data
+        sex_ratio (float): proportion of the population that is male (not currently used)
+        microstructure (bool): whether or not to use the microstructuring algorithm to group contacts
+
+    Returns:
+        popdict (dict): a dictionary representing the population, with the following keys for a population of N agents with M contacts between them:
+    '''
 
     pop_size = int(sim['pop_size']) # Number of people
 
@@ -143,20 +163,32 @@ def make_randpop(sim, use_age_data=True, use_household_data=True, sex_ratio=0.5,
     popdict['sex'] = sexes
 
     # Actually create the contacts
-    if   microstructure == 'random':    contacts, layer_keys = make_random_contacts(pop_size, sim['contacts'])
-    elif microstructure == 'clustered': contacts, layer_keys = make_microstructured_contacts(pop_size, sim['contacts'])
-    elif microstructure == 'hybrid':    contacts, layer_keys = make_hybrid_contacts(pop_size, ages, sim['contacts'])
+    if   microstructure == 'random':    contacts, layer_keys    = make_random_contacts(pop_size, sim['contacts'])
+    elif microstructure == 'clustered': contacts, layer_keys, _ = make_microstructured_contacts(pop_size, sim['contacts'])
+    elif microstructure == 'hybrid':    contacts, layer_keys, _ = make_hybrid_contacts(pop_size, ages, sim['contacts'])
     else:
         errormsg = f'Microstructure type "{microstructure}" not found; choices are random, clustered, or hybrid'
         raise NotImplementedError(errormsg)
 
-    popdict['contacts'] = contacts
+    popdict['contacts']   = contacts
+    popdict['layer_keys'] = layer_keys
 
-    return popdict, layer_keys
+    return popdict
 
 
-def make_random_contacts(pop_size, contacts, overshoot=1.2, directed=False):
-    ''' Make random static contacts '''
+def make_random_contacts(pop_size, contacts, overshoot=1.2):
+    '''
+    Make random static contacts.
+
+    Args:
+        pop_size (int): number of agents to create contacts between (N)
+        contacts (dict): a dictionary with one entry per layer describing the average number of contacts per person for that layer
+        overshoot (float): to avoid needing to take multiple Poisson draws
+
+    Returns:
+        contacts_list (list): a list of length N, where each entry is a dictionary by layer, and each dictionary entry is the UIDs of the agent's contacts
+        layer_keys (list): a list of layer keys, which is the same as the keys of the input "contacts" dictionary
+    '''
 
     # Preprocessing
     pop_size = int(pop_size) # Number of people
@@ -166,14 +198,11 @@ def make_random_contacts(pop_size, contacts, overshoot=1.2, directed=False):
 
     # Precalculate contacts
     n_across_layers = np.sum(list(contacts.values()))
-    n_all_contacts  = int(pop_size*n_across_layers*overshoot)
+    n_all_contacts  = int(pop_size*n_across_layers*overshoot) # The overshoot is used so we won't run out of contacts if the Poisson draws happen to be higher than the expected value
     all_contacts    = cvu.choose_r(max_n=pop_size, n=n_all_contacts) # Choose people at random
     p_counts = {}
     for lkey in layer_keys:
-        if directed:
-            p_counts[lkey] = cvu.n_poisson(contacts[lkey], pop_size)
-        else:
-            p_counts[lkey] = np.array((cvu.n_poisson(contacts[lkey], pop_size)/2.0).round(), dtype=cvd.default_int)  # Draw the number of Poisson contacts for this person
+        p_counts[lkey] = np.array((cvu.n_poisson(contacts[lkey], pop_size)/2.0).round(), dtype=cvd.default_int)  # Draw the number of Poisson contacts for this person
 
 
     # Make contacts
@@ -189,7 +218,7 @@ def make_random_contacts(pop_size, contacts, overshoot=1.2, directed=False):
     return contacts_list, layer_keys
 
 
-def make_microstructured_contacts(pop_size, contacts, directed=False):
+def make_microstructured_contacts(pop_size, contacts):
     ''' Create microstructured contacts -- i.e. households, schools, etc. '''
 
     # Preprocessing -- same as above
@@ -200,36 +229,36 @@ def make_microstructured_contacts(pop_size, contacts, directed=False):
     contacts_list = [{c:[] for c in layer_keys} for p in range(pop_size)] # Pre-populate
 
     for layer_name, cluster_size in contacts.items():
-        # Make clusters - each person belongs to one cluster
-        n_remaining = pop_size
+
+        # Initialize
+        cluster_dict = dict() # Add dictionary for this layer
+        n_remaining = pop_size # Make clusters - each person belongs to one cluster
         contacts_dict = defaultdict(set) # Use defaultdict of sets for convenience while initializing. Could probably change this as part of performance optimization
 
+        # Loop over the clusters
+        cluster_id = -1
         while n_remaining > 0:
-
-            # Get the size of this cluster
+            cluster_id += 1 # Assign cluster id
             this_cluster =  cvu.poisson(cluster_size)  # Sample the cluster size
             if this_cluster > n_remaining:
                 this_cluster = n_remaining
 
             # Indices of people in this cluster
             cluster_indices = (pop_size-n_remaining)+np.arange(this_cluster)
-
-            # Add symmetric pairwise contacts in each cluster. Can probably optimize this
-            for i in cluster_indices:
+            cluster_dict[cluster_id] = cluster_indices
+            for i in cluster_indices: # Add symmetric pairwise contacts in each cluster
                 for j in cluster_indices:
-                    if j <= i:
-                        pass
-                    else:
+                    if j > i:
                         contacts_dict[i].add(j)
-                        if directed:
-                            contacts_dict[j].add(i)
 
             n_remaining -= this_cluster
 
         for key in contacts_dict.keys():
             contacts_list[key][layer_name] = np.array(list(contacts_dict[key]), dtype=cvd.default_int)
 
-    return contacts_list, layer_keys
+        clusters = {layer_name: cluster_dict}
+
+    return contacts_list, layer_keys, clusters
 
 
 def make_hybrid_contacts(pop_size, ages, contacts, school_ages=None, work_ages=None):
@@ -252,7 +281,7 @@ def make_hybrid_contacts(pop_size, ages, contacts, school_ages=None, work_ages=N
     contacts_list = [{key:[] for key in layer_keys} for i in range(pop_size)]
 
     # Start with the household contacts for each person
-    h_contacts, _ = make_microstructured_contacts(pop_size, {'h':contacts['h']})
+    h_contacts, _, clusters = make_microstructured_contacts(pop_size, {'h':contacts['h']})
 
     # Make community contacts
     c_contacts, _ = make_random_contacts(pop_size, {'c':contacts['c']})
@@ -272,15 +301,36 @@ def make_hybrid_contacts(pop_size, ages, contacts, school_ages=None, work_ages=N
     for i,ind in enumerate(w_inds): contacts_list[ind]['w'] = w_inds[w_contacts[i]['w']] # Copy over work contacts
     for i     in range(pop_size):   contacts_list[i]['c']   =        c_contacts[i]['c']  # Copy over community contacts -- present for everyone
 
-    return contacts_list, layer_keys
+    return contacts_list, layer_keys, clusters
 
 
 
-def make_synthpop(sim, directed=False):
-    ''' Make a population using synthpops, including contacts '''
+def make_synthpop(sim, generate=True, layer_mapping=None, **kwargs):
+    '''
+    Make a population using SynthPops, including contacts. Usually called automatically,
+    but can also be called manually.
+
+    Args:
+        sim (Sim): a Covasim simulation object
+        generate (bool): whether or not to generate a new population (otherwise, tries to load a pre-generated one)
+        layer_mapping (dict): a custom mapping from SynthPops layers to Covasim layers
+        kwars (dict): passed to sp.make_population()
+
+    **Example**::
+
+        sim = cv.Sim(pop_type='synthpops')
+        sim.popdict = cv.make_synthpop(sim)
+        sim.run()
+    '''
     import synthpops as sp # Optional import
+
+    # Handle layer mapping
+    default_layer_mapping = {'H':'h', 'S':'s', 'W':'w', 'C':'c'} # Remap keys from old names to new names
+    layer_mapping = sc.mergedicts(default_layer_mapping, layer_mapping)
+
+    # Handle other input arguments
     pop_size = sim['pop_size']
-    population = sp.make_population(n=pop_size, generate=True)
+    population = sp.make_population(n=pop_size, generate=generate, **kwargs)
     uids, ages, sexes, contacts = [], [], [], []
     for uid,person in population.items():
         uids.append(uid)
@@ -289,18 +339,21 @@ def make_synthpop(sim, directed=False):
 
     # Replace contact UIDs with ints
     uid_mapping = {uid:u for u,uid in enumerate(uids)}
-    key_mapping = {'H':'h', 'S':'s', 'W':'w', 'C':'c'} # Remap keys from old names to new names
     for uid in uids:
         iid = uid_mapping[uid] # Integer UID
         person = population.pop(uid)
         uid_contacts = sc.dcp(person['contacts'])
         int_contacts = {}
         for spkey in uid_contacts.keys():
-            lkey = key_mapping[spkey] # Map the SynthPops key into a Covasim layer key
+            try:
+                lkey = layer_mapping[spkey] # Map the SynthPops key into a Covasim layer key
+            except KeyError:
+                errormsg = f'Could not find key "{spkey}" in layer mapping "{layer_mapping}"'
+                raise sc.KeyNotFoundError(errormsg)
             int_contacts[lkey] = []
             for cid in uid_contacts[spkey]: # Contact ID
                 icid = uid_mapping[cid] # Integer contact ID
-                if icid>iid or directed: # Don't add duplicate contacts
+                if icid>iid: # Don't add duplicate contacts
                     int_contacts[lkey].append(icid)
             int_contacts[lkey] = np.array(int_contacts[lkey], dtype=cvd.default_int)
         contacts.append(int_contacts)
@@ -312,10 +365,10 @@ def make_synthpop(sim, directed=False):
 
     # Finalize
     popdict = {}
-    popdict['uid']      = sc.dcp(uids)
-    popdict['age']      = np.array(ages)
-    popdict['sex']      = np.array(sexes)
-    popdict['contacts'] = sc.dcp(contacts)
-    layer_keys = list(key_mapping.values())
+    popdict['uid']        = sc.dcp(list(uid_mapping.values()))
+    popdict['age']        = np.array(ages)
+    popdict['sex']        = np.array(sexes)
+    popdict['contacts']   = sc.dcp(contacts)
+    popdict['layer_keys'] = list(layer_mapping.values())
 
-    return popdict, layer_keys
+    return popdict
