@@ -22,7 +22,7 @@ def make_metapars():
     ''' Create default metaparameters for a Scenarios run '''
     metapars = sc.objdict(
         n_runs    = 3, # Number of parallel runs; change to 3 for quick, 11 for real
-        noise     = 0.1, # Use noise, optionally
+        noise     = 0.0, # Use noise, optionally
         noisepar  = 'beta',
         rand_seed = 1,
         quantiles = {'low':0.1, 'high':0.9},
@@ -55,7 +55,7 @@ class MultiSim(sc.prettyobj):
         msim.plot() # Plot results
 
         sim = cv.Sim() # Create the sim
-        msim = cv.MultiSim(sim, n_runs=11, noise=0.1) # Set up a multisim with noise
+        msim = cv.MultiSim(sim, n_runs=11, noise=0.1, keep_people=True) # Set up a multisim with noise
         msim.run() # Run
         msim.reduce() # Compute statistics
         msim.plot() # Plot
@@ -114,7 +114,7 @@ class MultiSim(sc.prettyobj):
         Run the actual sims
 
         Args:
-            kwargs (dict): passed to multi_run() and thence to sim.run()
+            kwargs (dict): passed to multi_run() and thence (in part) to sim.run()
 
         Returns:
             None (modifies MultiSim object in place)
@@ -159,9 +159,14 @@ class MultiSim(sc.prettyobj):
             if not combined_sim.results[key].scale:
                 combined_sim.results[key].values /= n_runs
 
+        # Compute and store final results
+        combined_sim.compute_likelihood()
+        combined_sim.compute_summary(verbose=False)
         self.orig_base_sim = self.base_sim
         self.base_sim = combined_sim
         self.results = combined_sim.results
+        self.summary = combined_sim.summary
+
         self.which = 'combined'
 
         if output:
@@ -199,12 +204,14 @@ class MultiSim(sc.prettyobj):
             reduced_sim.results[reskey].values[:] = np.quantile(raw[reskey], q=0.5, axis=1) # Changed from median to mean for smoother plots
             reduced_sim.results[reskey].low       = np.quantile(raw[reskey], q=quantiles['low'],  axis=1)
             reduced_sim.results[reskey].high      = np.quantile(raw[reskey], q=quantiles['high'], axis=1)
-        reduced_sim.likelihood() # Recompute the likelihood for the average sim
-        reduced_sim.summary_stats(verbose=False) # Recalculate the summary stats
 
+        # Compute and store final results
+        reduced_sim.compute_likelihood()
+        reduced_sim.compute_summary(verbose=False)
         self.orig_base_sim = self.base_sim
         self.base_sim = reduced_sim
         self.results = reduced_sim.results
+        self.summary = reduced_sim.summary
         self.which = 'reduced'
 
         if output:
@@ -250,30 +257,56 @@ class MultiSim(sc.prettyobj):
         if do_plot:
             self.plot_compare(**kwargs)
 
+        df = pd.DataFrame.from_dict(resdict).astype(object) # astype is necessary to prevent type coercion
         if output:
-            df = pd.DataFrame.from_dict(resdict).astype(object) # astype is necessary to prevent type coersion
             return df
         else:
             print(df)
             return None
 
 
-    def plot(self, *args, **kwargs):
-        ''' Convenience method for plotting -- arguments passed to Sim.plot() '''
+    def plot(self, inds=None, alpha_range=None, plot_args=None, show_args=None, *args, **kwargs):
+        '''
+        Convenience method for plotting -- arguments passed to Sim.plot(). If
+        plotting multiple sims,
+
+        Args:
+            inds (list): if not combined or reduced, the indices of the simulations to plot (if None, plot all)
+            alpha_range (list): if not combined or reduced, a 2-element list/tuple/array providing the range of alpha values to use to distinguish the lines
+        '''
+        # Plot a single curve, possibly with a range
         if self.which in ['combined', 'reduced']:
             fig = self.base_sim.plot(*args, **kwargs)
+
+        # PLot individual lines
         else:
+
+            # Initialize
             fig = None
             orig_setylim = kwargs.get('setylim', True)
             kwargs['legend_args'] = sc.mergedicts({'show_legend':True}, kwargs.get('legend_args')) # Only plot the legend the first time
-            for s,sim in enumerate(self.sims):
-                if s != 0:
-                    kwargs['legend_args']['show_legend'] = False
-                if s == len(self.sims)-1:
+
+            # Handle indices and alpha
+            if inds is None:
+                inds = np.arange(len(self.sims))
+            if alpha_range is None:
+                alpha_range = [0.2, 0.8]
+            n_sims = len(inds)
+            alphas = np.linspace(alpha_range[0], alpha_range[1], n_sims)
+
+            # Plot
+            for s,ind in enumerate(inds):
+                sim = self.sims[ind]
+                if s > 0:
+                    show_args = False # Only show things like data the first time it's plotting
+                if s == len(inds)-1:
                     kwargs['setylim'] = orig_setylim
                 else:
                     kwargs['setylim'] = False
-                fig = sim.plot(fig=fig, *args, **kwargs)
+
+                plot_args = sc.mergedicts({'alpha':alphas[s]}, plot_args)
+                fig = sim.plot(fig=fig, plot_args=plot_args, show_args=show_args, *args, **kwargs)
+
         return fig
 
 
@@ -299,7 +332,8 @@ class MultiSim(sc.prettyobj):
 
     def plot_compare(self, t=-1, sim_inds=None, log_scale=True, **kwargs):
         '''
-        Plot a comparison between sims.
+        Plot a comparison between sims, using bars to show different values for
+        each result.
 
         Args:
             t (int): index of results, passed to compare()
@@ -313,6 +347,7 @@ class MultiSim(sc.prettyobj):
         df = self.compare(t=t, sim_inds=sim_inds, output=True)
         cvplt.plot_compare(df, log_scale=log_scale, **kwargs)
 
+
     def save(self, filename=None, keep_people=False, **kwargs):
         '''
         Save to disk as a gzipped pickle. Load with cv.load(filename).
@@ -361,7 +396,7 @@ class Scenarios(cvb.ParsObj):
     Args:
         sim (Sim or None): if supplied, use a pre-created simulation as the basis for the scenarios
         metapars (dict): meta-parameters for the run, e.g. number of runs; see make_metapars() for structure
-        scenarios (dict): a dictionary defining the scenarios; see default_scenario for structure
+        scenarios (dict): a dictionary defining the scenarios; see examples folder for examples; see below for baseline
         basepars (dict): a dictionary of sim parameters to be used for the basis of the scenarios (not required if sim is provided)
         scenfile (str): a filename for saving (defaults to the creation date)
 
@@ -382,7 +417,7 @@ class Scenarios(cvb.ParsObj):
             scenfile = f'covasim_scenarios_{datestr}.scens'
         self.scenfile = scenfile
 
-        # Handle scenarios -- by default, create a baseline scenario
+        # Handle scenarios -- by default, create the simplest possible baseline scenario
         if scenarios is None:
             scenarios = {'baseline':{'name':'Baseline', 'pars':{}}}
         self.scenarios = scenarios
@@ -535,7 +570,8 @@ class Scenarios(cvb.ParsObj):
             scatter_args (dict): Dictionary of kwargs to be passed to pl.scatter()
             axis_args    (dict): Dictionary of kwargs to be passed to pl.subplots_adjust()
             fill_args    (dict): Dictionary of kwargs to be passed to pl.fill_between()
-            legend_args  (dict): Dictionary of kwargs to be passed to pl.legend()
+            legend_args  (dict): Dictionary of kwargs to be passed to pl.legend(); if show_legend=False, do not show
+            show_args    (dict): Control which "extras" get shown: uncertainty bounds, data, interventions, ticks, and the legend
             as_dates     (bool): Whether to plot the x-axis as dates or time points
             dateformat   (str):  Date string format, e.g. '%B %d'
             interval     (int):  Interval between tick marks
@@ -693,7 +729,7 @@ class Scenarios(cvb.ParsObj):
         return scens
 
 
-def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, verbose=None, keep_people=False, run_args=None, sim_args=None, **kwargs):
+def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, keep_people=False, run_args=None, sim_args=None, verbose=None, **kwargs):
     '''
     Convenience function to perform a single simulation run. Mostly used for
     parallelization, but can also be used directly.
@@ -704,9 +740,10 @@ def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, verbose=None,
         reseed (bool): whether or not to generate a fresh seed for each run
         noise (float): the amount of noise to add to each run
         noisepar (string): the name of the parameter to add noise to
-        verbose (int): detail to print
+        keep_people (bool): whether to keep the people after the sim run
         run_args (dict): arguments passed to sim.run()
         sim_args (dict): extra parameters to pass to the sim, e.g. 'n_infected'
+        verbose (int): detail to print
         kwargs (dict): also passed to the sim
 
     Returns:
@@ -719,20 +756,18 @@ def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, verbose=None,
         sim = cv.single_run(sim) # Run it, equivalent(ish) to sim.run()
     '''
 
-    new_sim = sc.dcp(sim) # Copy the sim to avoid overwriting it
-
     # Set sim and run arguments
     sim_args = sc.mergedicts(sim_args, kwargs)
     run_args = sc.mergedicts({'verbose':verbose}, run_args)
     if verbose is None:
-        verbose = new_sim['verbose']
+        verbose = sim['verbose']
 
-    if not new_sim.label:
-        new_sim.label = f'Sim {ind:d}'
+    if not sim.label:
+        sim.label = f'Sim {ind:d}'
 
     if reseed:
-        new_sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
-        new_sim.set_seed()
+        sim['rand_seed'] += ind # Reset the seed, otherwise no point of parallel runs
+        sim.set_seed()
 
     # If the noise parameter is not found, guess what it should be
     if noisepar is None:
@@ -746,32 +781,32 @@ def single_run(sim, ind=0, reseed=True, noise=0.0, noisepar=None, verbose=None,
         noisefactor = 1 + noiseval
     else:
         noisefactor = 1/(1-noiseval)
-    new_sim[noisepar] *= noisefactor
+    sim[noisepar] *= noisefactor
 
     if verbose>=1:
-        print(f'Running a simulation using {new_sim["rand_seed"]} seed and {noisefactor} noise factor')
+        print(f'Running a simulation using {sim["rand_seed"]} seed and {noisefactor} noise factor')
 
     # Handle additional arguments
     for key,val in sim_args.items():
         print(f'Processing {key}:{val}')
-        if key in new_sim.pars.keys():
+        if key in sim.pars.keys():
             if verbose>=1:
-                print(f'Setting key {key} from {new_sim[key]} to {val}')
-                new_sim[key] = val
+                print(f'Setting key {key} from {sim[key]} to {val}')
+                sim[key] = val
         else:
             raise sc.KeyNotFoundError(f'Could not set key {key}: not a valid parameter name')
 
     # Run
-    new_sim.run(**run_args)
+    sim.run(**run_args)
 
     # Shrink the sim to save memory
     if not keep_people:
-        new_sim.shrink()
+        sim.shrink()
 
-    return new_sim
+    return sim
 
 
-def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=None, verbose=None, combine=False, keep_people=None, run_args=None, sim_args=None, par_args=None, **kwargs):
+def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=None, combine=False, keep_people=None, run_args=None, sim_args=None, par_args=None, verbose=None, **kwargs):
     '''
     For running multiple runs in parallel. If the first argument is a list of sims,
     exactly these will be run and most other arguments will be ignored.
@@ -783,12 +818,12 @@ def multi_run(sim, n_runs=4, reseed=True, noise=0.0, noisepar=None, iterpars=Non
         noise (float): the amount of noise to add to each run
         noisepar (string): the name of the parameter to add noise to
         iterpars (dict): any other parameters to iterate over the runs; see sc.parallelize() for syntax
-        verbose (int): detail to print
         combine (bool): whether or not to combine all results into one sim, rather than return multiple sim objects
-        keep_people (bool): whether or not to keep the people in each sim
+        keep_people (bool): whether to keep the people after the sim run
         run_args (dict): arguments passed to sim.run()
         sim_args (dict): extra parameters to pass to the sim
         par_args (dict): arguments passed to sc.parallelize()
+        verbose (int): detail to print
         kwargs (dict): also passed to the sim
 
     Returns:
