@@ -32,7 +32,8 @@ class Sim(cvb.BaseSim):
         label (str): the name of the simulation (useful to distinguish in batch runs)
         simfile (str): the filename for this simulation, if it's saved (default: creation date)
         popfile (str): the filename to load/save the population for this simulation
-        load_pop (bool): whether or not to load the population from the named file
+        load_pop (bool): whether to load the population from the named file
+        save_pop (bool): whether to save the population to the named file
         kwargs (dict): passed to make_pars()
 
     **Examples**::
@@ -41,9 +42,9 @@ class Sim(cvb.BaseSim):
         sim = cv.Sim(pop_size=10e3, datafile='my_data.xlsx')
     '''
 
-    def __init__(self, pars=None, datafile=None, datacols=None, label=None, simfile=None, popfile=None, load_pop=False, **kwargs):
+    def __init__(self, pars=None, datafile=None, datacols=None, label=None, simfile=None, popfile=None, load_pop=False, save_pop=False, **kwargs):
         # Create the object
-        default_pars = cvpar.make_pars(**kwargs) # Start with default pars
+        default_pars = cvpar.make_pars() # Start with default pars
         super().__init__(default_pars) # Initialize and set the parameters as attributes
 
         # Set attributes
@@ -52,6 +53,8 @@ class Sim(cvb.BaseSim):
         self.simfile       = simfile  # The filename of the sim
         self.datafile      = datafile # The name of the data file
         self.popfile       = popfile  # The population file
+        self.load_pop      = load_pop # Whether to load the population
+        self.save_pop      = save_pop # Whether to save the population
         self.data          = None     # The actual data
         self.popdict       = None     # The population dictionary
         self.t             = None     # The current time in the simulation
@@ -63,8 +66,8 @@ class Sim(cvb.BaseSim):
         # Now update everything
         self.set_metadata(simfile, label) # Set the simulation date and filename
         self.load_data(datafile, datacols) # Load the data, if provided
-        self.update_pars(pars)             # Update the parameters, if provided
-        if load_pop:
+        self.update_pars(pars, **kwargs)             # Update the parameters, if provided
+        if self.load_pop:
             self.load_population(popfile)      # Load the population, if provided
 
         return
@@ -74,9 +77,9 @@ class Sim(cvb.BaseSim):
         ''' Ensure that metaparameters get used properly before being updated '''
         pars = sc.mergedicts(pars, kwargs)
         if pars:
-            if 'pop_type' in pars:
-                cvpar.reset_layer_pars(pars)
-            if 'prog_by_age' in pars:
+            if pars.get('pop_type'):
+                cvpar.reset_layer_pars(pars, force=False)
+            if pars.get('prog_by_age'):
                 pars['prognoses'] = cvpar.get_prognoses(by_age=pars['prog_by_age']) # Reset prognoses
             super().update_pars(pars=pars, create=create) # Call update_pars() for ParsObj
         return
@@ -104,43 +107,99 @@ class Sim(cvb.BaseSim):
         return
 
 
-    def initialize(self, save_pop=False, load_pop=False, popfile=None, **kwargs):
+    def initialize(self, **kwargs):
         '''
-        Perform all initializations.
+        Perform all initializations, including validating the parameters, setting
+        the random number seed, creating the results structure, initializing the
+        people, validating the layer parameters (which requires the people),
+        and initializing the interventions.
 
         Args:
-            save_pop (bool): if true, save the population to popfile
-            load_pop (bool): if true, load the population from popfile
-            popfile (str): filename to load/save the population
             kwargs (dict): passed to init_people
         '''
         self.t = 0  # The current time index
         self.validate_pars() # Ensure parameters have valid values
         self.set_seed() # Reset the random seed
         self.init_results() # Create the results stucture
-        self.init_people(save_pop=save_pop, load_pop=load_pop, popfile=popfile, **kwargs) # Create all the people (slow)
+        self.init_people(save_pop=self.save_pop, load_pop=self.load_pop, popfile=self.popfile, **kwargs) # Create all the people (slow)
+        self.validate_layer_pars() # Once the population is initialized, validate the layer parameters again
         self.init_interventions()
         self.initialized = True
         return
 
 
-    def reset_layer_pars(self, force=True):
+    def layer_keys(self):
+        '''
+        Attempt to retrieve the current layer keys, in the following order: from
+        the people object (for an initialized sim), from the popdict (for one in
+        the process of being initialized), from the beta_layer parameter (for an
+        uninitialized sim), or by assuming a default (if none of the above are
+        available).
+        '''
+        try:
+            keys = list(self['beta_layer'].keys()) # Get keys from beta_layer since the "most required" layer parameter
+        except:
+            keys = []
+        return keys
+
+
+    def reset_layer_pars(self, layer_keys=None, force=False):
         '''
         Reset the parameters to match the population.
 
         Args:
-            force (bool): reset the pars even if they already exist
+            layer_keys (list): override the default layer keys (use stored keys by default)
+            force (bool): reset the parameters even if they already exist
         '''
-        if self.people is not None:
-            layer_keys = self.people.contacts.keys()
-        else:
-            layer_keys = None
+        if layer_keys is None:
+            if self.people is not None: # If people exist
+                layer_keys = self.people.contacts.keys()
+            elif self.popdict is not None:
+                layer_keys = self.popdict['layer_keys']
         cvpar.reset_layer_pars(self.pars, layer_keys=layer_keys, force=force)
         return
 
 
-    def validate_pars(self):
-        ''' Some parameters can take multiple types; this makes them consistent '''
+    def validate_layer_pars(self):
+        '''
+        Handle layer parameters, since they need to be validated after the population
+        creation, rather than before.
+        '''
+
+        # First, try to figure out what the layer keys should be and perform basic type checking
+        layer_keys = self.layer_keys()
+        layer_pars = cvpar.layer_pars # The names of the parameters that are specified by layer
+        for lp in layer_pars:
+            val = self[lp]
+            if sc.isnumber(val): # It's a scalar instead of a dict, assume it's all contacts
+                self[lp] = {k:val for k in layer_keys}
+
+        # Handle key mismaches
+        for lp in layer_pars:
+            lp_keys = set(self.pars[lp].keys())
+            if not lp_keys == set(layer_keys):
+                errormsg = f'Layer parameters have inconsistent keys with the layer keys {layer_keys}:'
+                for lp2 in layer_pars: # Fail on first error, but re-loop to list all of them
+                    errormsg += f'\n{lp2} = ' + ', '.join(self.pars[lp].keys())
+                raise sc.KeyNotFoundError(errormsg)
+
+        # Handle mismatches with the population
+        if self.people is not None:
+            pop_keys = set(self.people.contacts.keys())
+            if pop_keys != set(layer_keys):
+                errormsg = f'Please update your parameter keys {layer_keys} to match population keys {pop_keys}. You may find sim.reset_layer_pars() helpful.'
+                raise sc.KeyNotFoundError(errormsg)
+
+        return
+
+
+    def validate_pars(self, validate_layers=True):
+        '''
+        Some parameters can take multiple types; this makes them consistent.
+
+        Args:
+            validate_layers (bool): whether to validate layer parameters as well via validate_layer_pars() -- usually yes, except during initialization
+        '''
 
         # Handle types
         for key in ['pop_size', 'pop_infected', 'pop_size']:
@@ -175,28 +234,10 @@ class Sim(cvb.BaseSim):
                 errormsg = f'You must supply one of n_days and end_day, not "{n_days}" and "{end_day}"'
                 raise ValueError(errormsg)
 
-        # Handle contacts
-        contacts = self['contacts']
-        if sc.isnumber(contacts): # It's a scalar instead of a dict, assume it's all contacts
-            self['contacts']    = {'a':contacts}
-
-        # Handle key mismaches
-        beta_layer_keys = set(self.pars['beta_layer'].keys())
-        contacts_keys   = set(self.pars['contacts'].keys())
-        quar_eff_keys   = set(self.pars['quar_eff'].keys())
-        if not(beta_layer_keys == contacts_keys == quar_eff_keys):
-            errormsg = f'Layer parameters beta={beta_layer_keys}, contacts={contacts_keys}, quar_eff={quar_eff_keys} have inconsistent keys'
-            raise sc.KeyNotFoundError(errormsg)
-        if self.people is not None:
-            pop_keys = set(self.people.contacts.keys())
-            if pop_keys != beta_layer_keys:
-                errormsg = f'Please update your parameter keys {beta_layer_keys} to match population keys {pop_keys}. You may find sim.reset_layer_pars() helpful.'
-                raise sc.KeyNotFoundError(errormsg)
-
         # Handle population data
         popdata_choices = ['random', 'hybrid', 'clustered', 'synthpops']
         choice = self['pop_type']
-        if choice not in popdata_choices:
+        if choice and choice not in popdata_choices:
             choicestr = ', '.join(popdata_choices)
             errormsg = f'Population type "{choice}" not available; choices are: {choicestr}'
             raise ValueError(errormsg)
@@ -207,6 +248,10 @@ class Sim(cvb.BaseSim):
             if isinstance(interv, dict): # It's a dictionary representation of an intervention
                 self['interventions'][i] = cvi.InterventionDict(**interv)
 
+        # Optionally handle layer parameters
+        if validate_layers:
+            self.validate_layer_pars()
+
         return
 
 
@@ -238,18 +283,20 @@ class Sim(cvb.BaseSim):
         for key,label in cvd.result_stocks.items():
             self.results[f'n_{key}'] = init_res(label, color=dcols[key])
         self.results['n_susceptible'].scale = 'static'
-        self.results['bed_capacity']  = init_res('Bed demand relative to capacity', scale=False)
 
         # Other variables
+        self.results['prevalence']    = init_res('Prevalence', scale=False)
+        self.results['incidence']     = init_res('Incidence', scale=False)
         self.results['r_eff']         = init_res('Effective reproductive number', scale=False)
         self.results['doubling_time'] = init_res('Doubling time', scale=False)
+        self.results['test_yield']    = init_res('Testing yield', scale=False)
 
         # Populate the rest of the results
         if self['rescale']:
             scale = 1
         else:
             scale = self['pop_scale']
-        self.rescale_vec   = scale*np.ones(self.npts)
+        self.rescale_vec   = scale*np.ones(self.npts) # Not included in the results, but used to scale them
         self.results['t']    = self.tvec
         self.results['date'] = self.datevec
         self.results_ready   = False
@@ -264,6 +311,7 @@ class Sim(cvb.BaseSim):
 
         Args:
             popfile (str): name of the file to load
+            kwargs (dict): passed to sc.makefilepath()
         '''
         if popfile is None and self.popfile is not None:
             popfile = self.popfile
@@ -277,6 +325,7 @@ class Sim(cvb.BaseSim):
                 raise ValueError(errormsg)
             if self['verbose']:
                 print(f'Loaded population from {filepath}')
+            self.reset_layer_pars(force=False, layer_keys=self.popdict['layer_keys']) # Ensure that layer keys match the loaded population
         return
 
 
@@ -289,6 +338,7 @@ class Sim(cvb.BaseSim):
             load_pop (bool): if true, load the population from popfile
             popfile (str): filename to load/save the population
             verbose (int): detail to prnt
+            kwargs (dict): passed to cv.make_people()
         '''
 
         # Handle inputs
@@ -300,15 +350,12 @@ class Sim(cvb.BaseSim):
             self.load_population(popfile=popfile)
 
         # Actually make the people
-        cvpop.make_people(self, save_pop=save_pop, popfile=popfile, verbose=verbose, **kwargs)
-        self.people.initialize()
+        self.people = cvpop.make_people(self, save_pop=save_pop, popfile=popfile, verbose=verbose, **kwargs)
+        self.people.initialize() # Fully initialize the people
 
         # Create the seed infections
-        inds = np.arange(int(self['pop_infected']))
-        self.people.infect(inds=inds)
-        for ind in inds:
-            self.people.transtree.linelist[ind] = dict(source=None, target=ind, date=self.t, layer='seed_infection')
-
+        inds = cvu.choose(self['pop_size'], self['pop_infected'])
+        self.people.infect(inds=inds, layer='seed_infection')
         return
 
 
@@ -322,7 +369,7 @@ class Sim(cvb.BaseSim):
 
 
     def rescale(self):
-        ''' Dynamically rescale the population '''
+        ''' Dynamically rescale the population -- used during step() '''
         if self['rescale']:
             t = self.t
             pop_scale = self['pop_scale']
@@ -353,17 +400,16 @@ class Sim(cvb.BaseSim):
         # Perform initial operations
         self.rescale() # Check if we need to rescale
         people   = self.people # Shorten this for later use
-        flows    = people.update_states_pre(t=t) # Update the state of everyone and count the flows
+        people.update_states_pre(t=t) # Update the state of everyone and count the flows
         contacts = people.update_contacts() # Compute new contacts
-        bed_max  = people.count('severe') > self['n_beds'] if self['n_beds'] else False # Check for a bed constraint
+        hosp_max = people.count('severe')   > self['n_beds_hosp'] if self['n_beds_hosp'] else False # Check for acute bed constraint
+        icu_max  = people.count('critical') > self['n_beds_icu']  if self['n_beds_icu']  else False # Check for ICU bed constraint
 
         # Randomly infect some people (imported infections)
         n_imports = cvu.poisson(self['n_imports']) # Imported cases
         if n_imports>0:
-            imporation_inds = cvu.choose(max_n=len(people), n=n_imports)
-            flows['new_infections'] += people.infect(inds=imporation_inds, bed_max=bed_max)
-            for ind in imporation_inds:
-                self.people.transtree.linelist[ind] = dict(source=None, target=ind, date=self.t, layer='importation')
+            importation_inds = cvu.choose(max_n=len(people), n=n_imports)
+            people.infect(inds=importation_inds, hosp_max=hosp_max, icu_max=icu_max, layer='importation')
 
         # Apply interventions
         for intervention in self['interventions']:
@@ -371,12 +417,11 @@ class Sim(cvb.BaseSim):
         if self['interv_func'] is not None: # Apply custom intervention function
             self['interv_func'](self)
 
-        flows = people.update_states_post(flows) # Check for state changes after interventions
+        people.update_states_post() # Check for state changes after interventions
 
         # Compute the probability of transmission
         beta         = cvd.default_float(self['beta'])
         asymp_factor = cvd.default_float(self['asymp_factor'])
-        diag_factor  = cvd.default_float(self['diag_factor'])
         frac_time    = cvd.default_float(self['viral_dist']['frac_time'])
         load_ratio   = cvd.default_float(self['viral_dist']['load_ratio'])
         high_cap     = cvd.default_float(self['viral_dist']['high_cap'])
@@ -391,37 +436,30 @@ class Sim(cvb.BaseSim):
             betas   = layer['beta']
 
             # Compute relative transmission and susceptibility
-            rel_trans  = people.rel_trans
-            rel_sus    = people.rel_sus
-            inf        = people.infectious
-            sus        = people.susceptible
-            symp       = people.symptomatic
-            diag       = people.diagnosed
-            quar       = people.quarantined
-            quar_eff   = cvd.default_float(self['quar_eff'][lkey])
-            beta_layer = cvd.default_float(self['beta_layer'][lkey])
-            rel_trans, rel_sus = cvu.compute_trans_sus(rel_trans, rel_sus, inf, sus, beta_layer, viral_load, symp, diag, quar, asymp_factor, diag_factor, quar_eff)
+            rel_trans   = people.rel_trans
+            rel_sus     = people.rel_sus
+            inf         = people.infectious
+            sus         = people.susceptible
+            symp        = people.symptomatic
+            diag        = people.diagnosed
+            quar        = people.quarantined
+            iso_factor  = cvd.default_float(self['iso_factor'][lkey])
+            quar_factor = cvd.default_float(self['quar_factor'][lkey])
+            beta_layer  = cvd.default_float(self['beta_layer'][lkey])
+            rel_trans, rel_sus = cvu.compute_trans_sus(rel_trans, rel_sus, inf, sus, beta_layer, viral_load, symp, diag, quar, asymp_factor, iso_factor, quar_factor)
 
             # Calculate actual transmission
             for sources,targets in [[p1,p2], [p2,p1]]: # Loop over the contact network from p1->p2 and p2->p1
-                target_inds, edge_inds = cvu.compute_infections(beta, sources, targets, betas, rel_trans, rel_sus) # Calculate transmission!
-                flows['new_infections'] += people.infect(inds=target_inds, bed_max=bed_max) # Actually infect people
+                source_inds, target_inds = cvu.compute_infections(beta, sources, targets, betas, rel_trans, rel_sus) # Calculate transmission!
+                people.infect(inds=target_inds, hosp_max=hosp_max, icu_max=icu_max, source=source_inds, layer=lkey) # Actually infect people
 
-                # Store the transmission tree
-                for ind in edge_inds:
-                    source = sources[ind]
-                    target = targets[ind]
-                    transdict = dict(source=source, target=target, date=self.t, layer=lkey)
-                    self.people.transtree.linelist[target] = transdict
-                    self.people.transtree.targets[source].append(transdict)
 
         # Update counts for this time step: stocks
         for key in cvd.result_stocks.keys():
             self.results[f'n_{key}'][t] = people.count(key)
-        self.results['bed_capacity'][t] = self.results['n_severe'][t]/self['n_beds'] if self['n_beds'] else 0
 
         # Update counts for this time step: flows
-        for key,count in flows.items():
+        for key,count in people.flows.items():
             self.results[key][t] += count
 
         # Tidy up
@@ -429,14 +467,15 @@ class Sim(cvb.BaseSim):
         return
 
 
-    def run(self, do_plot=False, until=None, verbose=None, **kwargs):
+    def run(self, do_plot=False, until=None, verbose=None, restore_pars=True, **kwargs):
         '''
         Run the simulation.
 
         Args:
             do_plot (bool): whether to plot
             until (int): day to run until
-            verbose (int): level of detail to print
+            verbose (int): level of detail to print (otherwise uses self['verbose'])
+            restore_pars (bool): whether to make a copy of the parameters before the run and restore it after, so runs are repeatable
             kwargs (dict): passed to self.plot()
 
         Returns:
@@ -450,6 +489,8 @@ class Sim(cvb.BaseSim):
         else:
             self.validate_pars() # We always want to validate the parameters before running
             self.init_interventions() # And interventions
+        if restore_pars:
+            orig_pars = sc.dcp(self.pars) # Create a copy of the parameters, to restore after the run, in case they are dynamically modified
         if verbose is None:
             verbose = self['verbose']
         if until:
@@ -473,7 +514,7 @@ class Sim(cvb.BaseSim):
 
             # Check if we were asked to stop
             elapsed = sc.toc(T, output=True)
-            if elapsed > self['timelimit']:
+            if self['timelimit'] and elapsed > self['timelimit']:
                 sc.printv(f"Time limit ({self['timelimit']} s) exceeded", 1, verbose)
                 break
             elif self['stopping_func'] and self['stopping_func'](self):
@@ -485,7 +526,8 @@ class Sim(cvb.BaseSim):
         # End of time loop; compute cumulative results outside of the time loop
         self.finalize(verbose=verbose) # Finalize the results
         sc.printv(f'Run finished after {elapsed:0.2f} s.\n', 1, verbose)
-        self.summary = self.summary_stats(verbose=verbose)
+        if restore_pars:
+            self.pars = orig_pars # Restore the original parameters
         if do_plot: # Optionally plot
             self.plot(**kwargs)
 
@@ -508,9 +550,7 @@ class Sim(cvb.BaseSim):
         self.results['cum_infections'].values += self['pop_infected']*self.rescale_vec[0] # Include initially infected people
 
         # Perform calculations on results
-        self.compute_doubling()
-        self.compute_r_eff()
-        self.likelihood()
+        self.compute_results()
 
         # Convert results to a odicts/objdict to allow e.g. sim.results.diagnoses
         self.results = sc.objdict(self.results)
@@ -520,6 +560,41 @@ class Sim(cvb.BaseSim):
         return
 
 
+    def compute_results(self, verbose=None):
+        ''' Perform final calculations on the results '''
+        self.compute_prev_inci()
+        self.compute_yield()
+        self.compute_doubling()
+        self.compute_r_eff()
+        self.compute_likelihood()
+        self.compute_summary(verbose=verbose)
+        return
+
+
+    def compute_prev_inci(self):
+        '''
+        Compute prevalence and incidence. Prevalence is the current number of infected
+        people divided by the number of people who are alive. Incidence is the number
+        of new infections per day divided by the susceptible population.
+        '''
+        n_exposed = self.results['n_exposed'].values # Number of people currently infected
+        n_alive = self.scaled_pop_size - self.results['cum_deaths'].values # Number of people still alive
+        n_susceptible = self.results['n_susceptible'].values # Number of people still susceptible
+        new_infections = self.results['new_infections'].values # Number of new infections
+        self.results['prevalence'][:] = n_exposed/n_alive # Calculate the prevalence
+        self.results['incidence'][:] = new_infections/n_susceptible # Calculate the incidence
+        return
+
+
+    def compute_yield(self):
+        ''' Compute test yield -- number of positive tests divided by the total number of tests '''
+        n_diags = self.results['new_diagnoses'].values # Number of positive tests
+        n_tests = self.results['new_tests'].values # Total number of tests
+        inds = cvu.true(n_tests) # Pull out non-zero numbers of tests
+        self.results['test_yield'].values[inds] = n_diags[inds]/n_tests[inds] # Calculate the yield
+        return
+
+
     def compute_doubling(self, window=3, max_doubling_time=30):
         '''
         Calculate doubling time using exponential approximation -- a more detailed
@@ -570,26 +645,33 @@ class Sim(cvb.BaseSim):
         if method == 'daily':
 
             # Find the dates that everyone became infectious and recovered, and hence calculate infectious duration
-            recov_inds = self.people.defined('date_recovered')
-            dead_inds = self.people.defined('date_dead')
-            date_recov = self.people.date_recovered[recov_inds]
-            date_dead = self.people.date_dead[dead_inds]
+            recov_inds   = self.people.defined('date_recovered')
+            dead_inds    = self.people.defined('date_dead')
+            date_recov   = self.people.date_recovered[recov_inds]
+            date_dead    = self.people.date_dead[dead_inds]
             date_outcome = np.concatenate((date_recov, date_dead))
-            inds = np.concatenate((recov_inds, dead_inds))
-            date_inf = self.people.date_infectious[inds]
-            mean_inf = date_outcome.mean() - date_inf.mean()
+            inds         = np.concatenate((recov_inds, dead_inds))
+            date_inf     = self.people.date_infectious[inds]
+            mean_inf     = date_outcome.mean() - date_inf.mean()
 
             # Calculate R_eff as the mean infectious duration times the number of new infectious divided by the number of infectious people on a given day
-            values = mean_inf*self.results['new_infections'].values/(self.results['n_infectious'].values+1e-6)
-            if len(values) >= 3: # Can't smooth arrays shorter than this
-                values = sc.smooth(values, smoothing)
+            raw_values = mean_inf*self.results['new_infections'].values/(self.results['n_infectious'].values+1e-6)
+            if len(raw_values) >= 3: # Can't smooth arrays shorter than this since the default smoothing kernel has length 3
+                values = sc.smooth(raw_values, smoothing)
+                values[:smoothing] = raw_values[:smoothing] # To avoid numerical effects, replace the beginning and end with the original
+                values[-smoothing:] = raw_values[-smoothing:]
+            else:
+                values = raw_values
 
         # Alternate (traditional) method -- count from the date of infection or outcome
         elif method in ['infectious', 'outcome']:
 
+            # Store a mapping from each source to their date
+            source_dates = {}
+
             for t in self.tvec:
 
-                # Sources are easy -- count up the arrays
+                # Sources are easy -- count up the arrays for all the people who became infections on that day
                 if method == 'infectious':
                     inds = cvu.true(t == self.people.date_infectious) # Find people who became infectious on this timestep
                 elif method == 'outcome':
@@ -598,25 +680,29 @@ class Sim(cvb.BaseSim):
                     inds       = np.concatenate((recov_inds, dead_inds))
                 sources[t] = len(inds)
 
-                # Targets are hard -- loop over the transmission tree
+                # Create the mapping from sources to dates
                 for ind in inds:
-                    targets[t] += len(self.people.transtree.targets[ind])
+                    source_dates[ind] = t
+
+            # Targets are hard -- loop over the transmission tree
+            for transdict in self.people.infection_log:
+                source = transdict['source']
+                if source is not None and source in source_dates: # Skip seed infections and people with e.g. recovery after the end of the sim
+                    source_date = source_dates[source]
+                    targets[source_date] += 1
+
+                # for ind in inds:
+                #     targets[t] += len(self.people.transtree.targets[ind])
 
             # Populate the array -- to avoid divide-by-zero, skip indices that are 0
-            inds = sc.findinds(sources>0)
-            r_eff = np.zeros(self.npts)*np.nan
-            r_eff[inds] = targets[inds]/sources[inds]
+            r_eff = np.divide(targets, sources, out=np.full(self.npts, np.nan), where=sources > 0)
 
             # Use stored weights calculate the moving average over the window of timesteps, n
             num = np.nancumsum(r_eff * sources)
             num[window:] = num[window:] - num[:-window]
             den = np.cumsum(sources)
             den[window:] = den[window:] - den[:-window]
-
-            # Avoid dividing by zero
-            values = np.zeros(num.shape)*np.nan
-            ind = den > 0
-            values[ind] = num[ind]/den[ind]
+            values = np.divide(num, den, out=np.full(self.npts, np.nan), where=den > 0)
 
         # Method not recognized
         else:
@@ -643,19 +729,18 @@ class Sim(cvb.BaseSim):
         intervals2 = np.zeros(len(self.people))
         pos1 = 0
         pos2 = 0
-        targets = self.people.transtree.targets
         date_exposed = self.people.date_exposed
         date_symptomatic = self.people.date_symptomatic
-        for p in range(len(self.people)):
-            if len(targets[p])>0:
-                for target in targets[p]:
-                    target_ind = target['target']
-                    intervals1[pos1] = date_exposed[target_ind] - date_exposed[p]
-                    pos1 += 1
-                    if not np.isnan(date_symptomatic[p]):
-                        if not np.isnan(date_symptomatic[target_ind]):
-                            intervals2[pos2] = date_symptomatic[target_ind] - date_symptomatic[p]
-                            pos2 += 1
+
+        for infection in self.people.infection_log:
+            if infection['source'] is not None:
+                source_ind = infection['source']
+                target_ind = infection['target']
+                intervals1[pos1] = date_exposed[target_ind] - date_exposed[source_ind]
+                pos1 += 1
+                if np.isfinite(date_symptomatic[source_ind]) and np.isfinite(date_symptomatic[target_ind]):
+                    intervals2[pos2] = date_symptomatic[target_ind] - date_symptomatic[source_ind]
+                    pos2 += 1
 
         self.results['gen_time'] = {
                 'true':         np.mean(intervals1[:pos1]),
@@ -665,7 +750,7 @@ class Sim(cvb.BaseSim):
         return self.results['gen_time']
 
 
-    def likelihood(self, weights=None, verbose=None, eps=1e-16):
+    def compute_likelihood(self, weights=None, verbose=None, eps=1e-16):
         '''
         Compute the log-likelihood of the current simulation based on the number
         of new diagnoses.
@@ -714,7 +799,7 @@ class Sim(cvb.BaseSim):
         return loglike
 
 
-    def summary_stats(self, verbose=None):
+    def compute_summary(self, verbose=None):
         ''' Compute the summary statistics to display at the end of a run '''
 
         if verbose is None:
@@ -727,6 +812,7 @@ class Sim(cvb.BaseSim):
             if key.startswith('cum_'):
                 summary_str += f'   {summary[key]:5.0f} {self.results[key].name.lower()}\n'
         sc.printv(summary_str, 1, verbose)
+        self.summary = summary
 
         return summary
 
@@ -743,7 +829,8 @@ class Sim(cvb.BaseSim):
             plot_args    (dict): Dictionary of kwargs to be passed to pl.plot()
             scatter_args (dict): Dictionary of kwargs to be passed to pl.scatter()
             axis_args    (dict): Dictionary of kwargs to be passed to pl.subplots_adjust()
-            legend_args  (dict): Dictionary of kwargs to be passed to pl.legend()
+            legend_args  (dict): Dictionary of kwargs to be passed to pl.legend(); if show_legend=False, do not show
+            show_args    (dict): Control which "extras" get shown: uncertainty bounds, data, interventions, ticks, and the legend
             as_dates     (bool): Whether to plot the x-axis as dates or time points
             dateformat   (str):  Date string format, e.g. '%B %d'
             interval     (int):  Interval between tick marks
@@ -786,4 +873,4 @@ class Sim(cvb.BaseSim):
             sim.plot_result('r_eff')
         '''
         fig = cvplt.plot_result(sim=self, key=key, *args, **kwargs)
-        return fig
\ No newline at end of file
+        return fig
